""" Permissions Test for Workspace Rest API
"""
from django.test import SimpleTestCase
from mock.mock import patch
from rest_framework import status
from rest_framework.response import Response

import core_main_app.components.workspace.api as workspace_api
from core_main_app.rest.workspace import views as workspace_rest_views
from core_main_app.rest.workspace.serializers import WorkspaceSerializer
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestGetWorkspaceDetail(SimpleTestCase):
    """ Test Get Workspace Detail
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(workspace_rest_views.WorkspaceDetail.as_view(),
                                              None,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(WorkspaceSerializer, "data")
    def test_staff_returns_http_200(self, mock_workspace_api_get_by_id, mock_data_serializer_data):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_api_get_by_id.return_value = None
        mock_data_serializer_data.return_value = []

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.WorkspaceDetail.as_view(),
                                              user,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(WorkspaceSerializer, "data")
    def test_authenticated_returns_http_200(self, mock_workspace_api_get_by_id, mock_data_serializer_data):
        # Context
        user = create_mock_user('1')
        mock_workspace_api_get_by_id.return_value = None
        mock_data_serializer_data.return_value = []

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.WorkspaceDetail.as_view(),
                                              user,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDeleteWorkspace(SimpleTestCase):
    """ Test Delete Workspace
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_delete(workspace_rest_views.WorkspaceDetail.as_view(),
                                                 None,
                                                 param={'pk': 0})
        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(workspace_api, "delete")
    def test_staff_returns_http_204(self, mock_workspace_api_get_by_id, mock_workspace_api_delete):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_api_get_by_id.return_value = None
        mock_workspace_api_delete.returns_value = None

        # Act
        response = RequestMock.do_request_delete(workspace_rest_views.WorkspaceDetail.as_view(),
                                                 user,
                                                 param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(workspace_api, "delete")
    def test_authenticated_returns_http_204(self, mock_workspace_api_get_by_id, mock_workspace_api_delete):
        # Context
        user = create_mock_user('1')
        mock_workspace_api_get_by_id.return_value = None
        mock_workspace_api_delete.return_value = None

        # Act
        response = RequestMock.do_request_delete(workspace_rest_views.WorkspaceDetail.as_view(),
                                                 user,
                                                 param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestGetWorkspace(SimpleTestCase):
    """ Test Get Workspace
    """

    def test_get_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(workspace_rest_views.WorkspaceList.as_view(),
                                              None)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(WorkspaceSerializer, "data")
    def test_get_staff_returns_http_200(self, mock_workspace_api_get_by_id, mock_data_serializer_data):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_api_get_by_id.return_value = None
        mock_data_serializer_data.return_value = []

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.WorkspaceList.as_view(),
                                              user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(WorkspaceSerializer, "data")
    def test_get_authenticated_returns_http_200(self, mock_workspace_api_get_by_id, mock_data_serializer_data):
        # Context
        user = create_mock_user('1')
        mock_workspace_api_get_by_id.return_value = None
        mock_data_serializer_data.return_value = []

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.WorkspaceList.as_view(),
                                              user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestCreateWorkspace(SimpleTestCase):
    """ Test Create Workspace
    """
    def test_post_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_post(workspace_rest_views.WorkspaceList.as_view(),
                                               None,
                                               data={"title": "title 1"})
        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(WorkspaceSerializer, "data")
    @patch.object(WorkspaceSerializer, "is_valid")
    @patch.object(WorkspaceSerializer, "save")
    def test_post_staff_returns_http_201(self, mock_data_serializer_save, mock_data_serializer_is_valid, mock_data_serializer_data):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_data_serializer_is_valid.return_value = True
        mock_data_serializer_save.return_value = None
        mock_data_serializer_data.return_value = []

        # Act
        response = RequestMock.do_request_post(workspace_rest_views.WorkspaceList.as_view(),
                                               user,
                                               data={"title": "title 1"})
        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(WorkspaceSerializer, "data")
    @patch.object(WorkspaceSerializer, "is_valid")
    @patch.object(WorkspaceSerializer, "save")
    def test_post_authenticated_returns_http_201(self, mock_data_serializer_save, mock_data_serializer_is_valid, mock_data_serializer_data):
        # Context
        user = create_mock_user('1')
        mock_data_serializer_is_valid.return_value = True
        mock_data_serializer_save.return_value = None
        mock_data_serializer_data.return_value = []

        # Act
        response = RequestMock.do_request_post(workspace_rest_views.WorkspaceList.as_view(),
                                               user,
                                               data={"title": "title 1"})
        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestWorkspaceReadAccess(SimpleTestCase):
    """ Test Workspace Read Access
    """
    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_workspaces_with_read_access, None)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_api, "get_all_workspaces_with_read_access_by_user")
    @patch.object(workspace_rest_views, "_list_of_workspaces_to_response")
    def test_staff_returns_http_200(self, mock_workspace_rest_views__list_of_workspaces_to_response,
                                                                 mock_workspace_api_get_all_workspaces_with_read_access_by_user):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_api_get_all_workspaces_with_read_access_by_user.return_value = None
        mock_workspace_rest_views__list_of_workspaces_to_response.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_workspaces_with_read_access, user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_api, "get_all_workspaces_with_read_access_by_user")
    @patch.object(workspace_rest_views, "_list_of_workspaces_to_response")
    def test_authenticated_returns_http_200(self, mock_workspace_rest_views__list_of_workspaces_to_response,
                                                                 mock_workspace_api_get_all_workspaces_with_read_access_by_user):
        # Context
        user = create_mock_user('1')
        mock_workspace_api_get_all_workspaces_with_read_access_by_user.return_value = None
        mock_workspace_rest_views__list_of_workspaces_to_response.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_workspaces_with_read_access, user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWorkspaceWriteAccess(SimpleTestCase):
    """ Test Workspace Write Access
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_workspaces_with_write_access, None)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_api, "get_all_workspaces_with_write_access_by_user")
    @patch.object(workspace_rest_views, "_list_of_workspaces_to_response")
    def test_staff_returns_http_200(self, mock_workspace_rest_views__list_of_workspaces_to_response,
                                                                  mock_workspace_api_get_all_workspaces_with_write_access_by_user):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_api_get_all_workspaces_with_write_access_by_user.return_value = None
        mock_workspace_rest_views__list_of_workspaces_to_response.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_workspaces_with_write_access, user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_api, "get_all_workspaces_with_write_access_by_user")
    @patch.object(workspace_rest_views, "_list_of_workspaces_to_response")
    def test_authenticated_returns_http_200(self, mock_workspace_rest_views__list_of_workspaces_to_response,
                                                                  mock_workspace_api_get_all_workspaces_with_write_access_by_user):
        # Context
        user = create_mock_user('1')
        mock_workspace_api_get_all_workspaces_with_write_access_by_user.return_value = None
        mock_workspace_rest_views__list_of_workspaces_to_response.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_workspaces_with_write_access, user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWorkspaceIsPublic(SimpleTestCase):
    """ Test Workspace Is Public
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(workspace_rest_views.is_workspace_public,
                                              None,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(workspace_api, "is_workspace_public")
    def test_staff_returns_http_200(self, mock_workspace_api_get_by_id, mock_workspace_api_is_workspace_public):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_api_get_by_id.return_value = None
        mock_workspace_api_is_workspace_public.return_value = None

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.is_workspace_public,
                                              user,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(workspace_api, "is_workspace_public")
    def test_authenticated_returns_http_200(self, mock_workspace_api_get_by_id, mock_workspace_api_is_workspace_public):
        # Context
        user = create_mock_user('1')
        mock_workspace_api_get_by_id.return_value = None
        mock_workspace_api_is_workspace_public.return_value = None

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.is_workspace_public,
                                              user,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWorkspaceSetPublic(SimpleTestCase):
    """ Test Workspace Set Public
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.set_workspace_public,
                                                None,
                                                param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(workspace_api, "set_workspace_public")
    def test_staff_returns_http_200(self, mock_workspace_api_get_by_id, mock_workspace_api_set_workspace_public):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_api_get_by_id.return_value = None
        mock_workspace_api_set_workspace_public.return_value = None

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.set_workspace_public,
                                                user,
                                                param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(workspace_api, "set_workspace_public")
    def test_authenticated_returns_http_200(self, mock_workspace_api_get_by_id, mock_workspace_api_set_workspace_public):
        # Context
        user = create_mock_user('1')
        mock_workspace_api_get_by_id.return_value = None
        mock_workspace_api_set_workspace_public.return_value = None

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.set_workspace_public,
                                                user,
                                                param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWorkspaceSetPrivate(SimpleTestCase):
    """ Test Workspace Set Private
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.set_workspace_private,
                                                None,
                                                param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(workspace_api, "set_workspace_private")
    def test_staff_returns_http_200(self, mock_workspace_api_get_by_id, mock_workspace_api_set_workspace_private):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_api_get_by_id.return_value = None
        mock_workspace_api_set_workspace_private.return_value = None

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.set_workspace_private,
                                                user,
                                                param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_api, "get_by_id")
    @patch.object(workspace_api, "set_workspace_private")
    def test_authenticated_returns_http_200(self, mock_workspace_api_get_by_id, mock_workspace_api_set_workspace_private):
        # Context
        user = create_mock_user('1')
        mock_workspace_api_get_by_id.return_value = None
        mock_workspace_api_set_workspace_private.return_value = None

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.set_workspace_private,
                                                user,
                                                param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWorkspaceListUserCanRead(SimpleTestCase):
    """ Test Workspace List User Can read
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_list_user_can_read_workspace,
                                              None,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_rest_views, "_list_of_users_or_groups_to_response")
    def test_staff_returns_http_200(self, mock_workspace_rest_views__list_of_users_or_groups_to_response):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_rest_views__list_of_users_or_groups_to_response.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_list_user_can_read_workspace,
                                              user,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_rest_views, "_list_of_users_or_groups_to_response")
    def test_authenticated_returns_http_200(self, mock_workspace_rest_views__list_of_users_or_groups_to_response):
        # Context
        user = create_mock_user('1')
        mock_workspace_rest_views__list_of_users_or_groups_to_response.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_list_user_can_read_workspace,
                                              user,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWorkspaceListUserCanWrite(SimpleTestCase):
    """ Test Workspace List User Can Write
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_list_user_can_write_workspace,
                                              None,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_rest_views, "_list_of_users_or_groups_to_response")
    def test_staff_returns_http_200(self, mock_workspace_rest_views__list_of_users_or_groups_to_response):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_rest_views__list_of_users_or_groups_to_response.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_list_user_can_write_workspace,
                                              user,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_rest_views, "_list_of_users_or_groups_to_response")
    def test_authenticated_returns_http_200(self, mock_workspace_rest_views__list_of_users_or_groups_to_response):
        # Context
        user = create_mock_user('1')
        mock_workspace_rest_views__list_of_users_or_groups_to_response.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_list_user_can_write_workspace,
                                              user,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWorkspaceListGroupCanRead(SimpleTestCase):
    """ Test Workspace List Group Can read
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_list_group_can_read_workspace,
                                              None,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_rest_views, "_list_of_users_or_groups_to_response")
    def test_staff_returns_http_200(self, mock_workspace_rest_views__list_of_users_or_groups_to_response):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_rest_views__list_of_users_or_groups_to_response.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_list_group_can_read_workspace,
                                              user,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_rest_views, "_list_of_users_or_groups_to_response")
    def test_authenticated_returns_http_200(self, mock_workspace_rest_views__list_of_users_or_groups_to_response):
        # Context
        user = create_mock_user('1')
        mock_workspace_rest_views__list_of_users_or_groups_to_response.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_list_group_can_read_workspace,
                                              user,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestWorkspaceListGroupCanWrite(SimpleTestCase):
    """ Test Workspace List Group Can Write
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_list_group_can_write_workspace,
                                              None,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_rest_views, "_list_of_users_or_groups_to_response")
    def test_staff_returns_http_200(self, mock_workspace_rest_views__list_of_users_or_groups_to_response):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_rest_views__list_of_users_or_groups_to_response.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_list_group_can_write_workspace,
                                              user,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_rest_views, "_list_of_users_or_groups_to_response")
    def test_authenticated_returns_http_200(self, mock_workspace_rest_views__list_of_users_or_groups_to_response):
        # Context
        user = create_mock_user('1')
        mock_workspace_rest_views__list_of_users_or_groups_to_response.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_get(workspace_rest_views.get_list_group_can_write_workspace,
                                              user,
                                              param={'pk': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestAddUserReadRightToWorkspace(SimpleTestCase):
    """ Test Add User Read Right To Workspace
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.add_user_read_right_to_workspace,
                                                None,
                                                param={'pk':0, 'user_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace")
    def test_staff_returns_http_200(self, mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.add_user_read_right_to_workspace,
                                                user,
                                                param={'pk': 0, 'user_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace")
    def test_authenticated_returns_http_200(self, mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace):
        # Context
        user = create_mock_user('1')
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.add_user_read_right_to_workspace,
                                                user,
                                                param={'pk': 0, 'user_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestAddUserWriteRightToWorkspace(SimpleTestCase):
    """ Test Add User Write Right To Workspace
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.add_user_write_right_to_workspace,
                                                None,
                                                param={'pk': 0, 'user_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace")
    def test_staff_returns_http_200(self, mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.add_user_write_right_to_workspace,
                                                user,
                                                param={'pk': 0, 'user_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace")
    def test_authenticated_returns_http_200(self, mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace):
        # Context
        user = create_mock_user('1')
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.add_user_write_right_to_workspace,
                                                user,
                                                param={'pk': 0, 'user_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestAddGroupReadRightToWorkspace(SimpleTestCase):
    """ Test Add Group Read Right To Workspace
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.add_group_read_right_to_workspace,
                                                None,
                                                param={'pk': 0, 'group_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace")
    def test_staff_returns_http_200(self, mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.add_group_read_right_to_workspace,
                                                user,
                                                param={'pk': 0, 'group_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace")
    def test_authenticated_returns_http_200(self, mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace):
        # Context
        user = create_mock_user('1')
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.add_group_read_right_to_workspace,
                                                user,
                                                param={'pk': 0, 'group_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestAddGroupWriteRightToWorkspace(SimpleTestCase):
    """ Test Add Group Write Right To Workspace
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.add_group_write_right_to_workspace,
                                                None,
                                                param={'pk': 0, 'group_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace")
    def test_staff_returns_http_200(self, mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.add_group_write_right_to_workspace,
                                                user,
                                                param={'pk': 0, 'group_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace")
    def test_authenticated_returns_http_200(self, mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace):
        # Context
        user = create_mock_user('1')
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.add_group_write_right_to_workspace,
                                                user,
                                                param={'pk': 0, 'group_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestRemoveUserReadRightToWorkspace(SimpleTestCase):
    """ Test Remove User Read Right To Workspace
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.remove_user_read_right_to_workspace,
                                                None,
                                                param={'pk': 0, 'user_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace")
    def test_returns_http_200(self, mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.remove_user_read_right_to_workspace,
                                                user,
                                                param={'pk': 0, 'user_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace")
    def test_authenticated_returns_http_200(self, mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace):
        # Context
        user = create_mock_user('1')
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.remove_user_read_right_to_workspace,
                                                user,
                                                param={'pk': 0, 'user_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

class TestRemoveUserWriteRightToWorkspace(SimpleTestCase):
    """ Test Remove User Write Right To Workspace
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.remove_user_write_right_to_workspace,
                                                None,
                                                param={'pk': 0, 'user_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace")
    def test_staff_returns_http_200(self, mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.remove_user_write_right_to_workspace,
                                                user,
                                                param={'pk': 0, 'user_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace")
    def test_authenticated_returns_http_200(self, mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace):
        # Context
        user = create_mock_user('1')
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.remove_user_write_right_to_workspace,
                                                user,
                                                param={'pk': 0, 'user_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestRemoveGroupReadRightToWorkspace(SimpleTestCase):
    """ Test Remove Group Read Right To Workspace
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.remove_group_read_right_to_workspace,
                                                None,
                                                param={'pk': 0, 'group_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace")
    def test_staff_returns_http_200(self, mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.remove_group_read_right_to_workspace,
                                                user,
                                                param={'pk': 0, 'group_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace")
    def test_authenticated_returns_http_200(self, mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace):
        # Context
        user = create_mock_user('1')
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.remove_group_read_right_to_workspace,
                                                user,
                                                param={'pk': 0, 'group_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestRemoveGroupWriteRightToWorkspace(SimpleTestCase):
    """ Test Remove Group Write Right To Workspace
    """

    def test_anonymous_returns_http_403(self):
        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.remove_group_write_right_to_workspace,
                                                None,
                                                param={'pk': 0, 'group_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace")
    def test_staff_returns_http_200(self, mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace):
        # Context
        user = create_mock_user('1', is_staff=True)
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(
            status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.remove_group_write_right_to_workspace,
                                                user,
                                                param={'pk': 0, 'group_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(workspace_rest_views, "_add_or_remove_to_user_or_group_right_to_workspace")
    def test_authenticated_returns_http_200(self, mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace):
        # Context
        mock_workspace_rest_views__add_or_remove_to_user_or_group_right_to_workspace.return_value = Response(status=status.HTTP_200_OK)
        user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_patch(workspace_rest_views.remove_group_write_right_to_workspace,
                                                user,
                                                param={'pk': 0, 'group_id': 0})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


""" Common Serializers
"""
from rest_framework.serializers import Serializer


class BasicSerializer(Serializer):
    """ Represent a serializer with a basic implementation of the abstract methods create and update.
    """
    def create(self, validated_data):
        pass

    def update(self, instance, validated_data):
        pass



""" XslTransformation model
"""
from mongoengine import errors as mongoengine_errors

from core_main_app.commons import exceptions
from core_main_app.utils.validation.regex_validation import not_empty_or_whitespaces
from django_mongoengine import fields, Document


class XslTransformation(Document):
    """ XslTransformation object
    """
    name = fields.StringField(blank=False, unique=True, validation=not_empty_or_whitespaces)
    filename = fields.StringField(blank=False, validation=not_empty_or_whitespaces)
    content = fields.StringField(blank=False)

    meta = {'allow_inheritance': True}

    def __str__(self):
        """ String representation of an object.

        Returns:
            String representation

        """
        return self.name

    @staticmethod
    def get_all():
        """ Get all XSL Transformations.

        Returns:

        """
        return XslTransformation.objects.all()

    @staticmethod
    def get_by_name(xslt_name):
        """ Get XSL Transformation by name.

        Args:
            xslt_name:

        Returns:

        """
        try:
            return XslTransformation.objects.get(name=xslt_name)
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(str(e))
        except Exception as e:
            raise exceptions.ModelError(str(e))

    @staticmethod
    def get_by_id(xslt_id):
        """ Get an XSLT document by its id.

        Args:
            xslt_id: Id.

        Returns:
            XslTransformation object.

        """
        try:
            return XslTransformation.objects.get(pk=str(xslt_id))
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(str(e))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    def save_object(self):
        """ Custom save.

        Returns:
            Saved Instance.

        """
        try:
            return self.save()
        except mongoengine_errors.NotUniqueError as e:
            raise exceptions.NotUniqueError(str(e))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    def clean(self):
        """ Clean is called before saving

        Returns:

        """
        self.name = self.name.strip()
        self.filename = self.filename.strip()




""" Fixtures files for XslTransformation
"""
from core_main_app.components.xsl_transformation.models import XslTransformation
from core_main_app.utils.integration_tests.fixture_interface import FixtureInterface


class XslTransformationFixtures(FixtureInterface):
    """ Xsl Transformation fixtures
    """
    data_1 = None
    data_collection = None

    def insert_data(self):
        """ Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_data_collection()

    def generate_data_collection(self):
        """ Generate a Data collection.

        Returns:

        """
        content = '<?xml version=\"1.0\" encoding=\"UTF-8\"?>' \
                  '<xsl:stylesheet xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\" version=\"1.0\">' \
                  '<xsl:template></xsl:template></xsl:stylesheet>'
        self.data_1 = XslTransformation(name="name_1", filename='filename_1.xsd', content=content).save()
        self.data_collection = [self.data_1]



from django.conf.urls import include
from django.contrib import admin
from django.urls import re_path

from core_main_app import urls as core_main_app_urls

urlpatterns = [
    re_path(r'^admin/', include(admin.site.urls)),
] + core_main_app_urls.urlpatterns



"""Mailing util
"""
import core_main_app.utils.notifications.tasks.task_mail as task
from core_main_app.settings import SERVER_EMAIL, USE_BACKGROUND_TASK


def send_mail(recipient_list, subject, path_to_template, context={}, fail_silently=True, sender=SERVER_EMAIL):
    """Send email.

    Args:
        recipient_list:
        subject:
        path_to_template:
        context:
        fail_silently:
        sender:

    Returns:

    """
    if USE_BACKGROUND_TASK:
        # Async call. Use celery
        task.send_mail.apply_async((recipient_list, subject, path_to_template, context, fail_silently, sender), countdown=1)
    else:
        # Sync call
        task.send_mail(recipient_list, subject, path_to_template, context, fail_silently, sender)


def send_mail_to_administrators(subject, path_to_template, context={}, fail_silently=True):
    """Send email to administrators.

    Args:
        subject:
        path_to_template:
        context:
        fail_silently:

    Returns:

    """
    if USE_BACKGROUND_TASK:
        # Async call. Use celery
        task.send_mail_to_administrators.apply_async((subject, path_to_template, context, fail_silently), countdown=1)
    else:
        # Sync call
        task.send_mail_to_administrators(subject, path_to_template, context, fail_silently)


def send_mail_to_managers(subject, path_to_template, context={}, fail_silently=True):
    """Send email to managers.

    Args:
        subject:
        path_to_template:
        context:
        fail_silently:

    Returns:

    """
    if USE_BACKGROUND_TASK:
        # Async call. Use celery
        task.send_mail_to_managers.apply_async((subject, path_to_template, context, fail_silently), countdown=1)
    else:
        # Sync call
        task.send_mail_to_managers(subject, path_to_template, context, fail_silently)




""" Unit tests of get_attribute templatetag
"""
from unittest.case import TestCase

from core_main_app.templatetags.get_attribute import get_attribute


class TestObject(object):
    test = 'test'


class TestSystemGetAllExcept(TestCase):

    def test_get_attribute_on_empty_object(self):
        result = get_attribute({}, 'test')
        self.assertEqual(result, '')

    def test_get_attribute_on_none(self):
        result = get_attribute(None, 'test')
        self.assertEqual(result, '')

    def test_get_attribute_on_object(self):
        test_obj = TestObject()
        result = get_attribute(test_obj, 'test')
        self.assertEqual(result, 'test')

    def test_get_attribute_on_dict(self):
        test_dict = {'test': 'test'}
        result = get_attribute(test_dict, 'test')
        self.assertEqual(result, 'test')

    def test_get_attribute_on_table(self):
        test_tab = ['test']
        result = get_attribute(test_tab, '0')
        self.assertEqual(result, 'test')



""" Authentication tests for TemplateXSLRendering REST API
"""
from bson import ObjectId
from django.test import SimpleTestCase
from mock import patch, Mock
from rest_framework import status

from core_main_app.components.template_xsl_rendering import api as template_xsl_rendering_api
from core_main_app.rest.template_xsl_rendering import views as template_xsl_rendering_views
from core_main_app.rest.template_xsl_rendering.serializers import TemplateXslRenderingSerializer
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestTemplateXslRenderingListGetPermission(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            template_xsl_rendering_views.TemplateXslRenderingList.as_view(),
            None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(template_xsl_rendering_api, "get_all")
    def test_authenticated_returns_http_200(self, mock_template_xsl_rendering_api_get_all):
        mock_template_xsl_rendering_api_get_all.return_value = {}

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_get(
            template_xsl_rendering_views.TemplateXslRenderingList.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(template_xsl_rendering_api, "get_all")
    def test_staff_returns_http_200(self, mock_template_xsl_rendering_api_get_all):
        mock_template_xsl_rendering_api_get_all.return_value = {}

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_get(
            template_xsl_rendering_views.TemplateXslRenderingList.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestTemplateXslRenderingListPostPermission(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_post(
            template_xsl_rendering_views.TemplateXslRenderingList.as_view(),
            None,
            data={
                "template": str(ObjectId()),
                "list_xslt": str(ObjectId()),
                "detail_xslt": str(ObjectId())
            }
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(TemplateXslRenderingSerializer, "save")
    @patch.object(TemplateXslRenderingSerializer, "is_valid")
    @patch.object(TemplateXslRenderingSerializer, "data")
    def test_authenticated_returns_http_201(self, mock_template_xsl_rendering_serializer_data,
                                            mock_template_xsl_rendering_serializer_is_valid,
                                            mock_template_xsl_rendering_serializer_save):
        mock_template_xsl_rendering_serializer_save.return_value = None
        mock_template_xsl_rendering_serializer_is_valid.return_value = True
        mock_template_xsl_rendering_serializer_data.return_value = {}
        mock_user = create_mock_user('1')

        response = RequestMock.do_request_post(
            template_xsl_rendering_views.TemplateXslRenderingList.as_view(),
            mock_user,
            data={
                "template": str(ObjectId()),
                "list_xslt": str(ObjectId()),
                "detail_xslt": str(ObjectId())
            }
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(TemplateXslRenderingSerializer, "save")
    @patch.object(TemplateXslRenderingSerializer, "is_valid")
    @patch.object(TemplateXslRenderingSerializer, "data")
    def test_staff_returns_http_201(self, mock_template_xsl_rendering_serializer_data,
                                    mock_template_xsl_rendering_serializer_is_valid,
                                    mock_template_xsl_rendering_serializer_save):
        mock_template_xsl_rendering_serializer_save.return_value = None
        mock_template_xsl_rendering_serializer_is_valid.return_value = True
        mock_template_xsl_rendering_serializer_data.return_value = {}
        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_post(
            template_xsl_rendering_views.TemplateXslRenderingList.as_view(),
            mock_user,
            data={
                "template": str(ObjectId()),
                "list_xslt": str(ObjectId()),
                "detail_xslt": str(ObjectId())
            }
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestTemplateXslRenderingDetailGetPermission(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            template_xsl_rendering_views.TemplateXslRenderingDetail.as_view(),
            None,
            param={"pk": str(ObjectId())}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(template_xsl_rendering_api, "get_by_id")
    @patch.object(TemplateXslRenderingSerializer, "data")
    def test_authenticated_returns_http_200(self, mock_template_xsl_rendering_api_get_by_id,
                                            mock_template_xsl_serializer_data):
        mock_template_xsl_serializer_data.return_value = {}
        mock_template_xsl_rendering_api_get_by_id.return_value = Mock()
        mock_user = create_mock_user('1')

        response = RequestMock.do_request_get(
            template_xsl_rendering_views.TemplateXslRenderingDetail.as_view(),
            mock_user,
            param={"pk": str(ObjectId())}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(template_xsl_rendering_api, "get_by_id")
    @patch.object(TemplateXslRenderingSerializer, "data")
    def test_staff_returns_http_200(self, mock_template_xsl_rendering_api_get_by_id,
                                    mock_template_xsl_serializer_data):
        mock_template_xsl_serializer_data.return_value = {}
        mock_template_xsl_rendering_api_get_by_id.return_value = Mock()
        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_get(
            template_xsl_rendering_views.TemplateXslRenderingDetail.as_view(),
            mock_user,
            param={"pk": str(ObjectId())}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestTemplateXslRenderingDetailPatchPermission(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_patch(
            template_xsl_rendering_views.TemplateXslRenderingDetail.as_view(),
            None,
            param={"pk": str(ObjectId())},
            data={
                "list_xslt": str(ObjectId()),
                "detail_xslt": str(ObjectId())
            }
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(template_xsl_rendering_api, "get_by_id")
    @patch.object(TemplateXslRenderingSerializer, "save")
    @patch.object(TemplateXslRenderingSerializer, "is_valid")
    @patch.object(TemplateXslRenderingSerializer, "data")
    def test_authenticated_returns_http_200(self, mock_template_xsl_rendering_serializer_data,
                                            mock_template_xsl_rendering_serializer_is_valid,
                                            mock_template_xsl_rendering_serializer_save,
                                            mock_template_xsl_rendering_api_get_by_id):
        mock_template_xsl_rendering_api_get_by_id.return_value = {}
        mock_template_xsl_rendering_serializer_save.return_value = None
        mock_template_xsl_rendering_serializer_is_valid.return_value = True
        mock_template_xsl_rendering_serializer_data.return_value = {}
        mock_user = create_mock_user('1')

        response = RequestMock.do_request_patch(
            template_xsl_rendering_views.TemplateXslRenderingDetail.as_view(),
            mock_user,
            param={"pk": str(ObjectId())},
            data={
                "list_xslt": str(ObjectId()),
                "detail_xslt": str(ObjectId())
            }
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(template_xsl_rendering_api, "get_by_id")
    @patch.object(TemplateXslRenderingSerializer, "save")
    @patch.object(TemplateXslRenderingSerializer, "is_valid")
    @patch.object(TemplateXslRenderingSerializer, "data")
    def test_staff_returns_http_200(self, mock_template_xsl_rendering_serializer_data,
                                    mock_template_xsl_rendering_serializer_is_valid,
                                    mock_template_xsl_rendering_serializer_save,
                                    mock_template_xsl_rendering_api_get_by_id):
        mock_template_xsl_rendering_api_get_by_id.return_value = {}
        mock_template_xsl_rendering_serializer_save.return_value = None
        mock_template_xsl_rendering_serializer_is_valid.return_value = True
        mock_template_xsl_rendering_serializer_data.return_value = {}
        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_patch(
            template_xsl_rendering_views.TemplateXslRenderingDetail.as_view(),
            mock_user,
            param={"pk": str(ObjectId())},
            data={
                "list_xslt": str(ObjectId()),
                "detail_xslt": str(ObjectId())
            }
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestTemplateXslRenderingDetailDeletePermission(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_delete(
            template_xsl_rendering_views.TemplateXslRenderingDetail.as_view(),
            None,
            param={"pk": str(ObjectId())},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(template_xsl_rendering_api, "get_by_id")
    @patch.object(template_xsl_rendering_api, "delete")
    def test_authenticated_returns_http_204(self, mock_template_xsl_rendering_api_get_by_id,
                                            mock_template_xsl_rendering_api_delete):
        mock_template_xsl_rendering_api_get_by_id.return_value = Mock()
        mock_template_xsl_rendering_api_delete.return_value = None
        mock_user = create_mock_user('1')

        response = RequestMock.do_request_delete(
            template_xsl_rendering_views.TemplateXslRenderingDetail.as_view(),
            mock_user,
            param={"pk": str(ObjectId())}
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @patch.object(template_xsl_rendering_api, "get_by_id")
    @patch.object(template_xsl_rendering_api, "delete")
    def test_staff_returns_http_204(self, mock_template_xsl_rendering_api_get_by_id,
                                    mock_template_xsl_rendering_api_delete):
        mock_template_xsl_rendering_api_get_by_id.return_value = Mock()
        mock_template_xsl_rendering_api_delete.return_value = None
        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_delete(
            template_xsl_rendering_views.TemplateXslRenderingDetail.as_view(),
            mock_user,
            param={"pk": str(ObjectId())}
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)



""" Unit Test Validators
"""
import unittest

from django.core.exceptions import ValidationError

from core_main_app.commons.validators import UpperCaseLetterCountValidator, LowerCaseLetterCountValidator, \
    AlphabeticCharCountValidator, MaxOccurrenceCountValidator, NonAlphanumericCountValidator, DigitsCountValidator


class TestValidators(unittest.TestCase):
    def test_upper_case_validator_should_pass(self):

        try:
            uppercase_validator = UpperCaseLetterCountValidator(2)
            uppercase_validator.validate("passWordTest")
        except ValidationError as validation_error:
            self.fail("The validator should not raise a ValidationError.")
        except Exception as ex:
            self.fail("The validator should not raise an Exception.")

    def test_upper_case_validator_should_fail(self):
        uppercase_validator = UpperCaseLetterCountValidator(2)
        with self.assertRaises(ValidationError):
            uppercase_validator.validate("passwordtest")

    def test_lower_case_validator_should_pass(self):

        try:
            lowercase_validator = LowerCaseLetterCountValidator(2)
            lowercase_validator.validate("PaSSWoRDTeST")
        except ValidationError as validation_error:
            self.fail("The validator should not raise a ValidationError.")
        except Exception as ex:
            self.fail("The validator should not raise an Exception.")

    def test_lower_case_validator_should_fail(self):
        lowercase_validator = LowerCaseLetterCountValidator(2)
        with self.assertRaises(ValidationError):
            lowercase_validator.validate("PASSWORDTEST")

    def test_alphabetic_validator_should_pass(self):

        try:
            alpha_validator = AlphabeticCharCountValidator(5)
            alpha_validator.validate("!@#$t%^&j&*(y(*%t$#@A*")
        except ValidationError as validation_error:
            self.fail("The validator should not raise a ValidationError.")
        except Exception as ex:
            self.fail("The validator should not raise an Exception.")

    def test_alphabetic_validator_should_fail(self):
        alpha_validator = AlphabeticCharCountValidator(2)
        with self.assertRaises(ValidationError):
            alpha_validator.validate("!@#$%^&&*((*%$#@*")

    def test_max_occurrence_validator_should_pass(self):

        try:
            max_occur_validator = MaxOccurrenceCountValidator(5)
            max_occur_validator.validate("abbcccdddd")
        except ValidationError as validation_error:
            self.fail("The validator should not raise a ValidationError.")
        except Exception as ex:
            self.fail("The validator should not raise an Exception.")

    def test_max_occurrence_validator_should_fail(self):
        max_occur_validator = MaxOccurrenceCountValidator(2)
        with self.assertRaises(ValidationError):
            max_occur_validator.validate("abbcccddddeeeee")

    def test_non_alphanumeric_validator_should_pass(self):

        try:
            non_alphanumeric_validator = NonAlphanumericCountValidator(5)
            non_alphanumeric_validator.validate("abbccc!@dddd$%eeee*")
        except ValidationError as validation_error:
            self.fail("The validator should not raise a ValidationError.")
        except Exception as ex:
            self.fail("The validator should not raise an Exception.")

    def test_non_alphanumeric_validator_should_fail(self):
        non_alphanumeric_validator = NonAlphanumericCountValidator(5)
        with self.assertRaises(ValidationError):
            non_alphanumeric_validator.validate("abbc*ccdd@ddee#eee")

    def test_digits_validator_should_pass(self):

        try:
            digits_validator = DigitsCountValidator(5)
            digits_validator.validate("abbccc12345dddd")
        except ValidationError as validation_error:
            self.fail("The validator should not raise a ValidationError.")
        except Exception as ex:
            self.fail("The validator should not raise an Exception.")

    def test_digits_validator_should_fail(self):
        digits_validator = DigitsCountValidator(5)
        with self.assertRaises(ValidationError):
            digits_validator.validate("abbccc1234dddd")




""" REST views for the blob API
"""
from abc import abstractmethod, ABCMeta

from django.http import Http404
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.permissions import IsAuthenticated, IsAuthenticatedOrReadOnly, IsAdminUser
from rest_framework.response import Response
from rest_framework.views import APIView

import core_main_app.components.blob.api as blob_api
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons import exceptions
from core_main_app.components.workspace import api as workspace_api
from core_main_app.rest.blob.serializers import BlobSerializer, DeleteBlobsSerializer
from core_main_app.utils.file import get_file_http_response
from core_main_app.components.user import api as user_api


class AbstractBlobList(APIView, metaclass=ABCMeta):

    @abstractmethod
    def _get_blobs(self, request):
        """ Retrieve blobs

        Args:
            request:

        Returns:

        """
        raise NotImplementedError("_get_blobs method is not implemented.")

    def get(self, request):
        """ Get all Blob

        Url Parameters:

            filename: document_filename

        Examples:

            ../blob/
            ../blob?filename=[filename]

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: List of blob
            - code: 403
              content: Authentication error
            - code: 500
              content: Internal server error
        """
        try:
            blob_list = self._get_blobs(request)

            # Apply filters
            filename = self.request.query_params.get('filename', None)
            if filename is not None:
                blob_list = blob_list.filter(filename=filename)

            # Serialize object
            serializer = BlobSerializer(blob_list, many=True, context={'request': request})

            # Return response
            return Response(serializer.data, status=status.HTTP_200_OK)
        except AccessControlError as e:
            content = {'message': str(e)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {'message': str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class BlobListAdmin(AbstractBlobList):
    """ List all Blob
    """
    permission_classes = (IsAdminUser, )

    def _get_blobs(self, request):
        """ Retrieve blobs

        Args:
            request:

        Returns:

        """
        return blob_api.get_all(request.user)

    def get(self, request):
        """ Get all accessible Blob for admin user

        Url Parameters:

            filename: document_filename

        Examples:

            ../blob/
            ../blob?filename=[filename]

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: List of blob
            - code: 403
              content: Authentication error
            - code: 500
              content: Internal server error
        """
        return super(BlobListAdmin, self).get(request)


class BlobList(AbstractBlobList):
    """ List all user Blob, or create a new one
    """
    permission_classes = (IsAuthenticated, )

    def _get_blobs(self, request):
        """ Retrieve blobs

        Args:
            request:

        Returns:

        """
        return blob_api.get_all_by_user(user=request.user)

    def get(self, request):
        """ Get all blob by user

        Url Parameters:

            filename: document_filename

        Examples:

            ../blob/
            ../blob?filename=[filename]

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: List of blob
            - code: 403
              content: Authentication error
            - code: 500
              content: Internal server error
        """
        return super(BlobList, self).get(request)

    def post(self, request):
        """ Create Blob

        Parameters:

            {
                "blob": "[file]",
            }

        Code snippet:

            requests.post(url, files={'blob': open(BLOB_PATH, 'rb')}, auth=(USER, PSWD))

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: Created blob
            - code: 400
              content: Validation error
            - code: 500
              content: Internal server error
        """
        try:
            # Build serializer
            serializer = BlobSerializer(data=request.data, context={'request': request})

            # Validate data
            serializer.is_valid(True)

            # Save data
            serializer.save(user=request.user)

            # Return the serialized data
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        except ValidationError as validation_exception:
            content = {'message': validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {'message': str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class BlobDetail(APIView):
    """ Retrieve, update or delete a Blob
    """
    permission_classes = (IsAuthenticatedOrReadOnly, )

    def get_object(self, request, pk):
        """ Get Blob from db

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            Blob
        """
        try:
            return blob_api.get_by_id(pk, request.user)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """ Retrieve Blob

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 200
              content: Blob
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            blob_object = self.get_object(request, pk)

            # Serialize object
            serializer = BlobSerializer(blob_object, context={'request': request})

            # Return response
            return Response(serializer.data)
        except AccessControlError as e:
            content = {'message': str(e)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Http404:
            content = {'message': 'Blob not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def delete(self, request, pk):
        """ Delete Blob

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 204
              content: Deletion succeed
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            blob_object = self.get_object(request, pk)

            # delete object
            blob_api.delete(blob_object, request.user)

            return Response(status=status.HTTP_204_NO_CONTENT)
        except AccessControlError as e:
            content = {'message': str(e)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Http404:
            content = {'message': 'Blob not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class BlobDownload(APIView):
    """ Download Blob
    """

    def get_object(self, request, pk):
        """ Get Blob from db

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            Blob
        """
        try:
            return blob_api.get_by_id(pk, request.user)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """ Download the Blob file

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 200
              content: Blob file
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            blob_object = self.get_object(request, pk)

            return get_file_http_response(blob_object.blob, blob_object.filename)
        except AccessControlError as e:
            content = {'message': str(e)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Http404:
            content = {'message': 'Blob not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class BlobDeleteList(APIView):
    """ Delete a list of Blob
    """
    permission_classes = (IsAuthenticatedOrReadOnly, )

    def patch(self, request):
        """ Delete a list of Blob

        Parameters:

            [
                {
                    "id": "blob_id",
                },
                {
                    "id": "blob_id",
                }
            ]

        Args:

            request: HTTP request

        Returns:

            - code: 204
              content: Deletion succeed
            - code: 400
              content: Validation error
            - code: 403
              content: Authentication error
            - code: 500
              content: Internal server error
        """
        try:
            # Serialize data
            serializer = DeleteBlobsSerializer(data=request.data, many=True, context={'request': request})

            # Validate data
            serializer.is_valid(True)

            # Get list of unique ids
            blob_ids = set([blob['id'] for blob in serializer.validated_data])

            for blob_id in blob_ids:
                # Get blob with its id
                blob = blob_api.get_by_id(blob_id, request.user)
                # Delete blob
                blob_api.delete(blob, request.user)

            # Return the serialized data
            return Response(status=status.HTTP_204_NO_CONTENT)
        except ValidationError as validation_exception:
            content = {'message': validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except AccessControlError as access_control_error:
            content = {'message': str(access_control_error)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {'message': str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class BlobAssign(APIView):
    """ Assign a Blob to a Workspace.
    """
    permission_classes = (IsAuthenticated,)

    def get_object(self, request, pk):
        """ Get Blob from db

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            blob
        """
        try:
            return blob_api.get_by_id(pk, request.user)
        except exceptions.DoesNotExist:
            raise Http404

    def get_workspace(self, workspace_id):
        """ Retrieve a Workspace

        Args:

            workspace_id: ObjectId

        Returns:

            - code: 404
              content: Object was not found
        """
        try:
            return workspace_api.get_by_id(workspace_id)
        except exceptions.DoesNotExist:
            raise Http404

    def patch(self, request, pk, workspace_id):
        """ Assign Blob to a Workspace

        Args:

            request: HTTP request
            pk: ObjectId
            workspace_id: ObjectId

        Returns:

            - code: 200
              content: None
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            blob_object = self.get_object(request, pk)
            workspace_object = self.get_workspace(workspace_id)

            # Assign blob to workspace
            blob_api.assign(blob_object, workspace_object, request.user)
            return Response({}, status=status.HTTP_200_OK)
        except Http404:
            content = {'message': 'Blob or workspace not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except AccessControlError as ace:
            content = {'message': str(ace)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {'message': str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class BlobChangeOwner(APIView):
    """ Change the Owner of a blob
    """
    permission_classes = (IsAdminUser,)

    def get_object(self, request, pk):
        """ Get blob from db

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            Blob
        """
        try:
            return blob_api.get_by_id(pk, request.user)
        except exceptions.DoesNotExist:
            raise Http404

    def get_user(self, user_id):
        """ Retrieve a User

        Args:

            user_id: ObjectId

        Returns:

            - code: 404
              content: Object was not found
        """
        try:
            return user_api.get_user_by_id(user_id)
        except exceptions.DoesNotExist:
            raise Http404

    def patch(self, request, pk, user_id):
        """ Change the Owner of a blob

        Args:

            request: HTTP request
            pk: ObjectId
            user_id: ObjectId

        Returns:

            - code: 200
              content: None
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # get object
            data_object = self.get_object(request, pk)
            user_object = self.get_user(user_id)

            # change owner
            blob_api.change_owner(data_object, user_object, request.user)
            return Response({}, status=status.HTTP_200_OK)
        except Http404:
            content = {'message': 'Blob or user not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except AccessControlError as ace:
            content = {'message': str(ace)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {'message': str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


""" Authentication tests for Blob REST API
"""
from django.test import SimpleTestCase
from mock import Mock
from mock.mock import patch
from rest_framework import status

import core_main_app.components.blob.api as blob_api
from core_main_app.components.workspace import api as workspace_api
from core_main_app.components.blob.models import Blob
from core_main_app.rest.blob import views as blob_rest_views
from core_main_app.rest.blob.serializers import BlobSerializer, DeleteBlobsSerializer
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestBlobListAdminGetPermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            blob_rest_views.BlobListAdmin.as_view(),
            None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user('1')

        response = RequestMock.do_request_get(
            blob_rest_views.BlobListAdmin.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch('core_main_app.components.blob.api.get_all')
    def test_staff_returns_http_200(self, get_all):
        mock_user = create_mock_user('1', is_staff=True)
        response = RequestMock.do_request_get(
            blob_rest_views.BlobListAdmin.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestBlobListGetPermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            blob_rest_views.BlobList.as_view(),
            None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_200(self):
        mock_user = create_mock_user('1')

        response = RequestMock.do_request_get(
            blob_rest_views.BlobList.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_staff_returns_http_200(self):
        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_get(
            blob_rest_views.BlobList.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestBlobListPostPermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_post(
            blob_rest_views.BlobList.as_view(),
            None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(BlobSerializer, "data")
    @patch.object(BlobSerializer, "save")
    @patch.object(BlobSerializer, "is_valid")
    def test_authenticated_returns_http_201(self, mock_blob_serializer_is_valid, mock_blob_serializer_save,
                                            mock_blob_serializer_data):
        mock_blob_serializer_is_valid.return_value = True
        mock_blob_serializer_save.return_value = None
        mock_blob_serializer_data.return_value = []

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_post(
            blob_rest_views.BlobList.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(BlobSerializer, "data")
    @patch.object(BlobSerializer, "save")
    @patch.object(BlobSerializer, "is_valid")
    def test_staff_returns_http_201(self, mock_blob_serializer_is_valid, mock_blob_serializer_save,
                                    mock_blob_serializer_data):
        mock_blob_serializer_is_valid.return_value = True
        mock_blob_serializer_save.return_value = None
        mock_blob_serializer_data.return_value = []

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_post(
            blob_rest_views.BlobList.as_view(),
            mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestBlobDetailGetPermissions(SimpleTestCase):
    @patch.object(BlobSerializer, "data")
    @patch.object(blob_api, "get_by_id")
    def test_anonymous_returns_http_200(self, mock_blob_api_get_by_id, mock_blob_serializer_data):
        mock_blob_api_get_by_id.return_value = None
        mock_blob_serializer_data.return_value = []

        response = RequestMock.do_request_get(
            blob_rest_views.BlobDetail.as_view(),
            None,
            param={"pk": "0"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(BlobSerializer, "data")
    @patch.object(blob_api, "get_by_id")
    def test_authenticated_returns_http_200(self, mock_blob_api_get_by_id, mock_blob_serializer_data):
        mock_blob_api_get_by_id.return_value = []
        mock_blob_serializer_data.return_value = []

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_get(
            blob_rest_views.BlobDetail.as_view(),
            mock_user,
            param={"pk": "0"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(BlobSerializer, "data")
    @patch.object(blob_api, "get_by_id")
    def test_staff_returns_http_200(self, mock_blob_api_get_by_id, mock_blob_serializer_data):
        mock_blob_api_get_by_id.return_value = []
        mock_blob_serializer_data.return_value = []

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_get(
            blob_rest_views.BlobDetail.as_view(),
            mock_user,
            param={"pk": "0"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestBlobDetailDeletePermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_delete(
            blob_rest_views.BlobDetail.as_view(),
            None,
            param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(blob_api, "delete")
    @patch.object(blob_api, "get_by_id")
    def test_authenticated_returns_http_204(self, mock_blob_api_get_by_id, mock_blob_api_delete):
        mock_blob_api_get_by_id.return_value = None
        mock_blob_api_delete.return_value = None

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_delete(
            blob_rest_views.BlobDetail.as_view(),
            mock_user,
            param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @patch.object(blob_api, "delete")
    @patch.object(blob_api, "get_by_id")
    def test_staff_returns_http_204(self, mock_blob_api_get_by_id, mock_blob_api_delete):
        mock_blob_api_get_by_id.return_value = None
        mock_blob_api_delete.return_value = None

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_delete(
            blob_rest_views.BlobDetail.as_view(),
            mock_user,
            param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestBlobDownloadGetPermissions(SimpleTestCase):

    @patch.object(blob_api, "get_by_id")
    def test_anonymous_returns_http_200(self, mock_blob_api_get_by_id):
        mock_blob = Mock()
        mock_blob.blob = "blob_text"
        mock_blob.filename = "blob.txt"

        mock_blob_api_get_by_id.return_value = mock_blob

        response = RequestMock.do_request_get(
            blob_rest_views.BlobDownload.as_view(),
            None,
            param={"pk": "0"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(blob_api, "get_by_id")
    def test_authenticated_returns_http_200(self, mock_blob_api_get_by_id):
        mock_blob = Mock()
        mock_blob.blob = "blob_text"
        mock_blob.filename = "blob.txt"

        mock_blob_api_get_by_id.return_value = mock_blob

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_get(
            blob_rest_views.BlobDownload.as_view(),
            mock_user,
            param={"pk": "0"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(blob_api, "get_by_id")
    def test_staff_returns_http_200(self, mock_blob_api_get_by_id):
        mock_blob = Mock()
        mock_blob.blob = "blob_text"
        mock_blob.filename = "blob.txt"

        mock_blob_api_get_by_id.return_value = mock_blob

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_get(
            blob_rest_views.BlobDownload.as_view(),
            mock_user,
            param={"pk": "0"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestBlobDeleteListPatchPermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_patch(
            blob_rest_views.BlobDeleteList.as_view(),
            None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(DeleteBlobsSerializer, "validated_data")
    @patch.object(DeleteBlobsSerializer, "is_valid")
    def test_authenticated_returns_http_204(self, mock_blob_serializer_is_valid, mock_blob_serializer_validated_data):
        mock_blob_serializer_is_valid.return_value = True
        mock_blob_serializer_validated_data.return_value = list()

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_patch(
            blob_rest_views.BlobDeleteList.as_view(),
            mock_user,
            data=[]
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @patch.object(DeleteBlobsSerializer, "validated_data")
    @patch.object(DeleteBlobsSerializer, "is_valid")
    def test_staff_returns_http_204(self, mock_blob_serializer_is_valid, mock_blob_serializer_validated_data):
        mock_blob_serializer_is_valid.return_value = True
        mock_blob_serializer_validated_data.return_value = list()

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_patch(
            blob_rest_views.BlobDeleteList.as_view(),
            mock_user,
            data=[]
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestBlobAssignPatchPermissions(SimpleTestCase):
    @patch.object(blob_api, "assign")
    @patch.object(workspace_api, "get_by_id")
    @patch.object(blob_api, "get_by_id")
    def test_anonymous_returns_http_403(self, mock_data_api_get_by_id, mock_workspace_api_get_by_id,
                                        mock_data_api_assign):
        mock_data_api_get_by_id.return_value = None
        mock_workspace_api_get_by_id.return_value = None
        mock_data_api_assign.return_value = None

        response = RequestMock.do_request_patch(
            blob_rest_views.BlobAssign.as_view(),
            None,
            param={"pk": 0, "workspace_id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(blob_api, "assign")
    @patch.object(workspace_api, "get_by_id")
    @patch.object(blob_api, "get_by_id")
    def test_authenticated_returns_http_200(self, mock_blob_api_get_by_id, mock_workspace_api_get_by_id,
                                            mock_blob_api_assign):
        mock_blob_api_get_by_id.return_value = None
        mock_workspace_api_get_by_id.return_value = None
        mock_blob_api_assign.return_value = None

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_patch(
            blob_rest_views.BlobAssign.as_view(),
            mock_user,
            param={"pk": 0, "workspace_id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(blob_api, "assign")
    @patch.object(workspace_api, "get_by_id")
    @patch.object(blob_api, "get_by_id")
    def test_staff_returns_http_200(self, mock_blob_api_get_by_id, mock_workspace_api_get_by_id,
                                    mock_blob_api_assign):
        mock_blob_api_get_by_id.return_value = None
        mock_workspace_api_get_by_id.return_value = None
        mock_blob_api_assign.return_value = None

        mock_user = create_mock_user('1', is_staff=True)

        response = RequestMock.do_request_patch(
            blob_rest_views.BlobAssign.as_view(),
            mock_user,
            param={"pk": 0, "workspace_id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestBlobChangeOwnerPatchPermissions(SimpleTestCase):

    @patch.object(blob_api, "change_owner")
    @patch('core_main_app.components.user.api.get_user_by_id')
    @patch.object(Blob, 'get_by_id')
    def test_anonymous_returns_http_403(self,
                                        mock_blob_api_get_by_id,
                                        mock_user_api_get_by_id,
                                        mock_blob_api_change_owner):
        mock_blob_api_get_by_id.return_value = None
        mock_user_api_get_by_id.return_value = None
        mock_blob_api_change_owner.return_value = None

        response = RequestMock.do_request_patch(
            blob_rest_views.BlobChangeOwner.as_view(),
            None,
            param={"pk": 0, "user_id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(blob_api, "change_owner")
    @patch('core_main_app.components.user.api.get_user_by_id')
    @patch.object(Blob, 'get_by_id')
    def test_authenticated_returns_http_403(self,
                                            mock_blob_api_get_by_id,
                                            mock_user_api_get_by_id,
                                            mock_blob_api_change_owner):
        mock_blob_api_get_by_id.return_value = None
        mock_user_api_get_by_id.return_value = None
        mock_blob_api_change_owner.return_value = None

        mock_user = create_mock_user('1')

        response = RequestMock.do_request_patch(
            blob_rest_views.BlobChangeOwner.as_view(),
            mock_user,
            param={"pk": 0, "user_id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(blob_api, "change_owner")
    @patch('core_main_app.components.user.api.get_user_by_id')
    @patch.object(Blob, 'get_by_id')
    def test_staff_returns_http_200(self,
                                    mock_blob_api_get_by_id,
                                    mock_user_api_get_by_id,
                                    mock_blob_api_change_owner):
        # Arrange
        # is_staff to access the view
        # is_superuser to be able to change the owner
        user_request = create_mock_user("1", is_staff=True, is_superuser=True)
        mock_blob_api_get_by_id.return_value = None
        mock_user_api_get_by_id.return_value = None
        mock_blob_api_change_owner.return_value = None

        # Act
        response = RequestMock.do_request_patch(
            blob_rest_views.BlobChangeOwner.as_view(),
            user_request,
            param={"pk": 0, "user_id": 0}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


""" SSL utils test class
"""
from unittest import TestCase

from core_main_app.commons.exceptions import SSLError
from core_main_app.utils.requests_utils.ssl import check_ssl_certificates_dir_setting


class TestCheckSslCertificatesDirSetting(TestCase):

    def test_check_ssl_certificates_dir_setting_ok_when_set_to_True(self):
        check_ssl_certificates_dir_setting(True)

    def test_check_ssl_certificates_dir_setting_ok_when_set_to_False(self):
        check_ssl_certificates_dir_setting(False)

    def test_check_ssl_certificates_dir_setting_raises_error_if_bad_parameter_type(self):
        with self.assertRaises(SSLError):
            check_ssl_certificates_dir_setting(1)

    def test_check_ssl_certificates_dir_setting_fails_when_set_to_bad_location(self):
        fake_dir_path = './fake/dir'
        with self.assertRaises(SSLError):
            check_ssl_certificates_dir_setting(fake_dir_path)










from os.path import join, dirname, realpath
from unittest.case import TestCase

from mock.mock import Mock, patch

import core_main_app.components.xsl_transformation.api as xsl_transformation_api
from core_main_app.commons import exceptions
from core_main_app.components.xsl_transformation.models import XslTransformation
from xml_utils.html_tree.parser import html_diff as htmldiff


class TestXslTransformationGet(TestCase):

    @patch.object(XslTransformation, 'get_by_name')
    def test_xsl_transformation_get_return_xsl_transformation(self, mock_get_by_name):
        # Arrange
        mock_xslt = _create_mock_xsl_transformation(name="xslt_name",
                                                    filename="xslt_filename",
                                                    content="xslt_content")

        mock_get_by_name.return_value = mock_xslt

        # Act
        result = xsl_transformation_api.get_by_name(mock_xslt.name)

        # Assert
        self.assertIsInstance(result, XslTransformation)

    @patch.object(XslTransformation, 'get_by_name')
    def test_xsl_transformation_get_raises_exception_if_object_does_not_exists(self, mock_get_by_name):
        # Arrange
        mock_unexisting_name = "unexisting_xslt_name"
        mock_get_by_name.side_effect = Exception()

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            xsl_transformation_api.get_by_name(mock_unexisting_name)


class TestXslTransformationGetById(TestCase):

    @patch.object(XslTransformation, 'get_by_id')
    def test_xsl_transformation_get_by_id_raises_api_error_if_does_not_exisst(self, mock_get):
        # Arrange
        mock_get.side_effect = exceptions.DoesNotExist('')

        # Act + Assert
        with self.assertRaises(exceptions.DoesNotExist):
            xsl_transformation_api.get_by_id(1)

    @patch.object(XslTransformation, 'get_by_id')
    def test_xsl_transformation_get_by_id_returns_xsl_transformation(self, mock_get):
        # Arrange
        mock_data = _create_mock_xsl_transformation(name="xslt_name_1",
                                                    filename="xslt_filename_1",
                                                    content="xslt_content_1")
        mock_get.return_value = mock_data

        # Act
        result = xsl_transformation_api.get_by_id(1)

        # Assert
        self.assertIsInstance(result, XslTransformation)


class TestXslTransformationGetAll(TestCase):

    @patch.object(XslTransformation, 'get_all')
    def test_xsl_transformation_get_all_contains_only_xsl_transformation(self, mock_get_all):
        # Arrange
        mock_xslt_1 = _create_mock_xsl_transformation(name="xslt_name_1",
                                                      filename="xslt_filename_1",
                                                      content="xslt_content_1")

        mock_xslt_2 = _create_mock_xsl_transformation(name="xslt_name_2",
                                                      filename="xslt_filename_2",
                                                      content="xslt_content_2")

        mock_get_all.return_value = [mock_xslt_1, mock_xslt_2]

        # Act
        result = xsl_transformation_api.get_all()

        # Assert
        self.assertTrue(all(isinstance(item, XslTransformation) for item in result))


class TestXslTransformationUpsert(TestCase):

    @patch.object(XslTransformation, 'save')
    def test_xsl_transformation_upsert_return_xsl_transformation(self, mock_save):
        # Arrange
        mock_name = "xslt_name"
        mock_filename = "xslt_filename"
        mock_content = "<xsl:stylesheet xmlns:xsl='http://www.w3.org/1999/XSL/Transform' version='1.0'> " \
                       "<xsl:output method='html' indent='yes' encoding='UTF-8' /><root></root></xsl:stylesheet>"

        mock_xslt = _create_xsl_transformation(name=mock_name,
                                               filename=mock_filename,
                                               content=mock_content)

        mock_save.return_value = mock_xslt

        # Act
        result = xsl_transformation_api.upsert(mock_xslt)

        # Assert
        self.assertIsInstance(result, XslTransformation)

    @patch.object(XslTransformation, 'save')
    def test_xsl_transformation_upsert_raises_exception_if_not_well_formatted(self, mock_save):
        # Arrange
        mock_name = "xslt_name"
        mock_filename = "xslt_filename"
        mock_content = "bad_content"

        mock_xslt = _create_xsl_transformation(name=mock_name,
                                               filename=mock_filename,
                                               content=mock_content)

        mock_save.return_value = mock_xslt

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            xsl_transformation_api.upsert(mock_xslt)

    @patch.object(XslTransformation, 'save')
    def test_xsl_transformation_upsert_raises_exception_if_bad_namespace(self, mock_save):
        # Arrange
        mock_name = "xslt_name"
        mock_filename = "xslt_filename"
        mock_content = "<root></root>"

        mock_xslt = _create_xsl_transformation(name=mock_name,
                                               filename=mock_filename,
                                               content=mock_content)

        mock_save.return_value = mock_xslt

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            xsl_transformation_api.upsert(mock_xslt)


class TestXslTransform(TestCase):

    @patch.object(XslTransformation, 'get_by_name')
    def test_xsl_transform_return_expected_string(self, mock_get_by_name):
        # Arrange
        mock_data_path = join(dirname(realpath(__file__)), "data")
        mock_xml_path = join(mock_data_path, "data.xml")
        mock_xsl_path = join(mock_data_path, "transform.xsl")
        mock_html_path = join(mock_data_path, "data_transformed.html")

        with open(mock_xml_path, "r") as xml_file:
            mock_xml_data = xml_file.read()

        mock_xslt = _create_mock_xsl_transformation(name="mock_xslt",
                                                    filename="mock_xslt.xsl")

        with open(mock_xsl_path, "r") as xsl_file:
            mock_xslt.content = xsl_file.read()

        mock_get_by_name.return_value = mock_xslt

        with open(mock_html_path, "r") as html_file:
            expected_result = html_file.read()

        # Act
        result = xsl_transformation_api.xsl_transform(mock_xml_data, mock_xslt.name)
        html_diff = htmldiff(result, expected_result)  # Computing difference in resulting content

        # Assert
        self.assertNotIn("<ins>", html_diff)
        self.assertNotIn("<del>", html_diff)

    @patch.object(XslTransformation, 'get_by_name')
    def test_xsl_transform_raise_api_error_on_encode_exception(self, mock_get_by_name):
        # Arrange
        mock_data_path = join(dirname(realpath(__file__)), "data")
        mock_xml_path = join(mock_data_path, "data.xml")

        with open(mock_xml_path, "r") as xml_file:
            mock_xml_data = xml_file.read()

        # two .encode() in a row will trigger the exception
        mock_xslt = _create_mock_xsl_transformation(name="mock_xslt",
                                                    filename="mock_xslt.xsl",
                                                    content=u"\u2000".encode("utf-8"))

        mock_get_by_name.return_value = mock_xslt

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            xsl_transformation_api.xsl_transform(mock_xml_data, mock_xslt.name)

    @patch.object(XslTransformation, 'get_by_name')
    def test_xsl_transform_raise_api_error_on_malformed_xslt(self, mock_get_by_name):
        # Arrange
        mock_data_path = join(dirname(realpath(__file__)), "data")
        mock_xml_path = join(mock_data_path, "data.xml")

        with open(mock_xml_path, "r") as xml_file:
            mock_xml_data = xml_file.read()

        mock_xslt = _create_mock_xsl_transformation(name="mock_xslt",
                                                    filename="mock_xslt.xsl",
                                                    content="mock_malformed_xslt/>")
        mock_get_by_name.return_value = mock_xslt

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            xsl_transformation_api.xsl_transform(mock_xml_data, mock_xslt.name)

    @patch.object(XslTransformation, 'get_by_name')
    def test_xsl_transform_raise_api_error_on_malformed_xml(self, mock_get_by_name):
        # Arrange
        mock_data_path = join(dirname(realpath(__file__)), "data")
        mock_xsl_path = join(mock_data_path, "transform.xsl")

        mock_xml_data = "<tag>Unclosed"

        mock_xslt = _create_mock_xsl_transformation(name="mock_xslt",
                                                    filename="mock_xslt.xsl")

        with open(mock_xsl_path, "r") as xsl_file:
            mock_xslt.content = xsl_file.read()

        mock_get_by_name.return_value = mock_xslt

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            xsl_transformation_api.xsl_transform(mock_xml_data, mock_xslt.name)

    @patch.object(XslTransformation, 'get_by_name')
    @patch('core_main_app.utils.xml.xsl_transform')
    def test_xsl_transform_raise_api_error_on_other_exception(self, mock_xsl_transform, mock_get_by_name):
        # Arrange
        mock_data_path = join(dirname(realpath(__file__)), "data")
        mock_xml_path = join(mock_data_path, "data.xml")
        mock_xsl_path = join(mock_data_path, "transform.xsl")

        with open(mock_xml_path, "r") as xml_file:
            mock_xml_data = xml_file.read()

        mock_xslt = _create_mock_xsl_transformation(name="mock_xslt",
                                                    filename="mock_xslt.xsl")

        with open(mock_xsl_path, "r") as xsl_file:
            mock_xslt.content = xsl_file.read()

        mock_get_by_name.return_value = mock_xslt
        mock_xsl_transform.side_effect = Exception()

        # Act + Assert
        with self.assertRaises(exceptions.ApiError):
            xsl_transformation_api.xsl_transform(mock_xml_data, mock_xslt.name)


def _create_mock_xsl_transformation(name=None, filename=None, content=None):
    mock_xslt = Mock(spec=XslTransformation)
    mock_xslt.name = name
    mock_xslt.filename = filename
    mock_xslt.content = content
    return mock_xslt


def _create_xsl_transformation(name=None, filename=None, content=None):
    xsl_transformation = XslTransformation()
    xsl_transformation.name = name
    xsl_transformation.filename = filename
    xsl_transformation.content = content
    return xsl_transformation



""" Utils to provide regex validation
"""
import re

from mongoengine import ValidationError

from core_main_app.commons.regex import NOT_EMPTY_OR_WHITESPACES


def not_empty_or_whitespaces(value):
    """ Check that the value is not an empty string or only white spaces

    Args:
        value:

    Returns:

    """
    pattern = re.compile(NOT_EMPTY_OR_WHITESPACES)
    if not pattern.match(value):
        raise ValidationError('Value can not be empty')



"""Web page API
"""
import core_main_app.commons.exceptions as exceptions
from core_main_app.commons.enums import WEB_PAGE_TYPES
from core_main_app.components.web_page.models import WebPage


def get(page_type):
    """Get the web page of a given type

        Args:
            page_type: type of the web page

        Returns: web page corresponding to the given id
    """
    if page_type not in list(WEB_PAGE_TYPES.keys()):
        return None
    try:
        return WebPage.get_by_type(page_type)
    except exceptions.DoesNotExist:
        return None


def delete_by_type(page_type):
    """ Delete the web pages with the given type

    Args:
        page_type (int): page type

    Returns: Web page

    """
    return WebPage.delete_by_type(page_type)


def upsert(web_page):
    """ Upsert the page content

        Args:
            web_page (obj): web page object

        Returns: content of the web page
    """
    if web_page.type not in list(WEB_PAGE_TYPES.values()):
        raise exceptions.ApiError("Web page type does not exist")

    # strip in case of whitespaces only
    if web_page.content.strip():
        # we save the object only if the content is not empty
        return web_page.save()
    else:
        # otherwise it means deletion
        return delete_by_type(web_page.type)



""" Tests of the web page API
"""

from unittest.case import TestCase

from mock import Mock, patch

from core_main_app.commons.enums import WEB_PAGE_TYPES
from core_main_app.commons.exceptions import ApiError
from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.web_page import api as web_page_api
from core_main_app.components.web_page.models import WebPage


class TestsWebPageApiGet(TestCase):

    @patch('core_main_app.components.web_page.models.WebPage.get_by_type')
    def test_web_page_get_login(self, mock_get_web_page_by_type):
        # Arrange
        content = "content web page login"
        mock_get_web_page_by_type.return_value = _create_mock_web_page(WEB_PAGE_TYPES["login"], content)
        # Act
        result = web_page_api.get("login")
        # Assert
        self.assertEqual("content web page login", result.content)

    @patch('core_main_app.components.web_page.models.WebPage.get_by_type')
    def test_web_page_get_not_in_database_return_none(self, mock_get_web_page_by_type):
        # Arrange
        mock_get_web_page_by_type.side_effect = DoesNotExist
        # Act
        result = web_page_api.get("fake_type")
        # Assert
        self.assertEqual(None, result)

    def test_web_page_get_with_wrong_type_return_none(self):
        # Arrange
        # Act
        page = web_page_api.get("wrong_type")
        # Assert
        self.assertEqual(None, page)


class TestsWebPageApiUpsert(TestCase):

    def test_web_page_upsert_type_does_not_exist(self):
        # Arrange
        web_page = WebPage(4, "test")
        # Act # Assert
        with self.assertRaises(ApiError):
            web_page_api.upsert(web_page)

    @patch('core_main_app.components.web_page.models.WebPage.save')
    def test_web_page_upsert_type_exist(self, mock_save):
        # Arrange
        web_page_type = WEB_PAGE_TYPES["login"]
        content = "content"
        web_page = WebPage(web_page_type, content)
        mock_save.return_value = WebPage(web_page_type, content)
        # Act
        result = web_page_api.upsert(web_page)
        # Assert
        self.assertEquals(content, result.content)


def _create_mock_web_page(page_type=-1, content="content"):
    mock_web_page = Mock(spec=WebPage)
    mock_web_page.type = page_type
    mock_web_page.content = content
    return mock_web_page



""" Url router for the main application
"""
from django.contrib.auth import views as auth_views
from django.contrib.auth.decorators import login_required
from drf_yasg.views import get_schema_view
from drf_yasg import openapi
from django.conf.urls import include
from django.urls import re_path

from core_main_app.components.blob import api as blob_api
from core_main_app.components.data import api as data_api
from core_main_app.utils.rendering import render
from core_main_app.views.common import ajax as common_ajax, views as common_views
from core_main_app.views.user import views as user_views, ajax as user_ajax

schema_view = get_schema_view(openapi.Info(title="REST API", default_version="v1",),)

urlpatterns = [
    re_path(r"^$", user_views.homepage, name="core_main_app_homepage"),
    re_path(r"^login", user_views.custom_login, name="core_main_app_login"),
    re_path(r"^logout", user_views.custom_logout, name="core_main_app_logout"),
    re_path(r"^locked", common_views.defender_error_page, name="core_main_app_locked"),
    re_path(r"^rest/", include("core_main_app.rest.urls")),
    re_path(
        r"^data", common_views.ViewData.as_view(), name="core_main_app_data_detail"
    ),
    re_path(
        r"^template/versions/(?P<version_manager_id>\w+)",
        user_views.manage_template_versions,
        name="core_main_app_manage_template_versions",
    ),
    re_path(
        r"^template/(?P<pk>[\w-]+)/edit/$",
        login_required(common_ajax.EditTemplateVersionManagerView.as_view()),
        name="core_main_app_edit_template",
    ),
    re_path(
        r"^template/disable",
        common_ajax.disable_version_manager,
        name="core_main_app_disable_template",
    ),
    re_path(
        r"^template/restore",
        common_ajax.restore_version_manager,
        name="core_main_app_restore_template",
    ),
    re_path(
        r"^template/version/disable",
        common_ajax.disable_template_version_from_version_manager,
        name="core_main_app_disable_template_version",
    ),
    re_path(
        r"^template/version/restore",
        common_ajax.restore_template_version_from_version_manager,
        name="core_main_app_restore_template_version",
    ),
    re_path(
        r"^template/version/current",
        common_ajax.set_current_template_version_from_version_manager,
        name="core_main_app_set_current_template_version",
    ),
    re_path(r"^xslt$", common_views.XSLTView.as_view(), name="core_main_app_xslt"),
    re_path(
        r"^xslt/upload$",
        login_required(common_views.UploadXSLTView.as_view()),
        name="core_main_app_upload_xslt",
    ),
    re_path(
        r"^template/xslt/(?P<template_id>\w+)",
        login_required(
            common_views.TemplateXSLRenderingView.as_view(
                rendering=render,
                template_name="core_main_app/common/templates_xslt/main.html",
                save_redirect="core_main_app_manage_template_versions",
            )
        ),
        name="core_main_app_template_xslt",
    ),
    re_path(
        r"^edit-rights/(?P<workspace_id>\w+)$",
        login_required(common_views.EditWorkspaceRights.as_view()),
        name="core_main_edit_rights_workspace",
    ),
    re_path(
        r"^create-workspace",
        user_ajax.create_workspace,
        name="core_main_create_workspace",
    ),
    re_path(
        r"^change-workspace",
        login_required(user_ajax.LoadFormChangeWorkspace.as_view()),
        name="core_main_change_workspace",
    ),
    re_path(
        r"^assign-blob-workspace",
        login_required(user_ajax.AssignView.as_view(api=blob_api)),
        name="core_main_assign_blob_workspace",
    ),
    re_path(
        r"^assign-data-workspace",
        login_required(user_ajax.AssignView.as_view(api=data_api)),
        name="core_main_assign_data_workspace",
    ),
    re_path(
        r"^public-workspace",
        user_ajax.set_public_workspace,
        name="core_main_public_workspace",
    ),
    re_path(
        r"^private-workspace",
        user_ajax.set_private_workspace,
        name="core_main_private_workspace",
    ),
    re_path(
        r"^add-user-form",
        user_ajax.load_add_user_form,
        name="core_main_edit_rights_users_form",
    ),
    re_path(
        r"^add-user-right-to-workspace",
        user_ajax.add_user_right_to_workspace,
        name="core_main_add_user_right_to_workspace",
    ),
    re_path(r"^switch-right", user_ajax.switch_right, name="core_main_switch_right"),
    re_path(
        r"^remove-rights",
        user_ajax.remove_user_or_group_rights,
        name="core_main_remove_rights",
    ),
    re_path(
        r"^add-group-form",
        user_ajax.load_add_group_form,
        name="core_main_edit_rights_groups_form",
    ),
    re_path(
        r"^add-group-right-to-workspace",
        user_ajax.add_group_right_to_workspace,
        name="core_main_add_group_right_to_workspace",
    ),
    re_path(
        r"^docs/api$",
        schema_view.with_ui("swagger", cache_timeout=0),
        name="swagger_view",
    ),
    re_path(r"^tz_detect/", include("tz_detect.urls")),
    re_path(
        r"^password_reset/$", user_views.custom_reset_password, name="password_reset"
    ),
    re_path(
        r"^password_reset/done/$",
        user_views.custom_password_reset_done,
        name="password_reset_done",
    ),
    re_path(
        r"^reset/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$",
        user_views.custom_password_reset_confirm,
        name="password_reset_confirm",
    ),
    re_path(
        r"^reset/done/$",
        user_views.custom_password_reset_complete,
        name="password_reset_complete",
    ),
]



""" REST views for the template XSL rendering API
"""
from django.http import Http404
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from core_main_app.commons import exceptions
from core_main_app.components.template_xsl_rendering import api as template_xsl_rendering_api
from core_main_app.rest.template_xsl_rendering.serializers import TemplateXslRenderingSerializer
from core_main_app.access_control.exceptions import AccessControlError


class TemplateXslRenderingList(APIView):
    """ List all template XSL renderings, or create a new one
    """
    permission_classes = (IsAuthenticated, )

    def get(self, request):
        """ Get all templates XSL renderings

        Args:
            request: HTTP request

        Returns:
            - code: 200
              content: List of XSL renderings
            - code: 401
              content: Unauthorized
            - code: 500
              content: Internal server error
        """
        try:
            # Retrieve all objects
            template_xsl_rendering_list = template_xsl_rendering_api.get_all()

            # Serialize object
            serializer = TemplateXslRenderingSerializer(template_xsl_rendering_list, many=True)

            # Return response
            return Response(serializer.data, status=status.HTTP_200_OK)
        except Exception as api_exception:
            content = {'message': str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def post(self, request):
        """ Create a XSL template rendering

        Parameters:
            {
                "template": "template_object_id",
                "list_xslt": "xslt_object_id",
                "detail_xslt": "xslt_object_id"
            }

        Args:
            request: HTTP request

        Returns:
            - code: 201
              content: XSL rendering created
            - code: 400
              content: Validation error / not unique / model error
            - code: 500
              content: Internal server error
        """
        try:
            # Build serializer
            serializer = TemplateXslRenderingSerializer(data=request.data)

            # Validate data
            serializer.is_valid(True)

            # Save data
            serializer.save()

            return Response(serializer.data, status=status.HTTP_201_CREATED)
        except ValidationError as validation_exception:
            content = {'message': validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.ModelError as validation_exception:
            content = {'message': str(validation_exception)}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.NotUniqueError as validation_exception:
            content = {'message': str(validation_exception)}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except KeyError as validation_exception:
            content = {'message': validation_exception}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {'message': str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class TemplateXslRenderingDetail(APIView):
    """ TemplateXslRendering details view
    """
    permission_classes = (IsAuthenticated,)

    def get(self, request, pk):
        """ Get `TemplateXSLRendering` object from db

        Args:
            request: HTTP request
            pk: ObjectId

        Returns:
            TemplateXSLRendering
        """
        try:
            # Get object
            template_xsl_rendering_object = template_xsl_rendering_api.get_by_id(pk)

            # Serialize object
            template_xsl_rendering_serializer = TemplateXslRenderingSerializer(
                template_xsl_rendering_object
            )

            # Return response
            return Response(template_xsl_rendering_serializer.data)
        except exceptions.DoesNotExist:
            content = {'message': 'XSL rendering object not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def patch(self, request, pk):
        """ Edit `TemplateXSLRendering` object from db

        Args:
            request: HTTP request
            pk: ObjectId

        Returns:
            TemplateXSLRendering
        """
        try:
            # Get object
            template_xsl_rendering_object = template_xsl_rendering_api.get_by_id(pk)

            # Build serializer
            template_xsl_rendering_serializer = TemplateXslRenderingSerializer(
                instance=template_xsl_rendering_object,
                data=request.data
            )

            # Validate data
            template_xsl_rendering_serializer.is_valid(True)
            # Save data
            template_xsl_rendering_serializer.save()

            return Response(template_xsl_rendering_serializer.data, status=status.HTTP_200_OK)
        except ValidationError as validation_exception:
            content = {'message': validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Http404:
            content = {'message': 'Data not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def delete(self, request, pk):
        """ Delete a TemplateXSLRendering

        Args:
            request: HTTP request
            pk: ObjectId

        Returns:
            - code: 204
              content: Deletion successful
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            template_xsl_rendering_object = template_xsl_rendering_api.get_by_id(pk)

            # delete object
            template_xsl_rendering_api.delete(template_xsl_rendering_object)

            # Return response
            return Response(status=status.HTTP_204_NO_CONTENT)
        except exceptions.DoesNotExist:
            content = {'message': 'Template XSL rendering not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except AccessControlError as ace:
            content = {'message': str(ace)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {'message': str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



from django import forms

from core_main_app.commons.validators import BlankSpacesValidator


class UploadForm(forms.Form):
    """
    Form to upload.
    """

    name = forms.CharField(max_length=100, required=True, widget=forms.TextInput(attrs={'class': 'form-control'}),
                           validators=[BlankSpacesValidator()])
    upload_file = forms.FileField(label='Select a file', required=True)



""" Rendering utils
"""
from os.path import splitext

from django.contrib.staticfiles import finders
from django.shortcuts import render as django_render


def _build_js_assets(js_assets):
    """Build js assets structure

    Args:
        js_assets:

    Returns:

    """
    updated_js_assets = []
    included_js_paths = [asset["path"] for asset in js_assets]

    for asset in js_assets:
        # By default an asset is not raw javascript
        if "is_raw" not in asset:
            asset["is_raw"] = False

        # By default asset are stored in the static dir
        if "is_external" not in asset:
            asset["is_external"] = False

        if not asset["is_raw"]:  # If an asset is not raw js, we look for raw js child file
            js_asset_splitext = splitext(asset["path"])
            js_raw_file = js_asset_splitext[0] + ".raw" + js_asset_splitext[1]

            # Test that the file exists and is not already included
            if finders.find(js_raw_file) is not None and js_raw_file not in included_js_paths:
                updated_js_assets.append(
                    {
                        "path": js_raw_file,
                        "is_raw": True
                    }
                )

        updated_js_assets.append(asset)

    return updated_js_assets


def _render(request, template_name, wrapper_name, modals=None, assets=None, context=None, use_theme=True):
    """Render a selected template.

    Args:
        request:
        template_name:
        wrapper_name:
        modals:
        assets:
        context:
        use_theme:

    Returns:

    """
    modals = modals if modals is not None else []
    assets = assets if assets is not None else {"css": [], "js": []}
    js_asset = assets["js"] if "js" in assets else []
    assets["js"] = _build_js_assets(js_asset)
    context = context if context is not None else {}

    # Rebuild the context for the app_wrapper template
    context = {
        "template": template_name,
        "modals": modals,
        "assets": assets,
        "data": context,
        "use_theme": use_theme,
    }

    return django_render(request, wrapper_name, context)


def render(request, template_name, modals=None, assets=None, context=None, use_theme=True):
    """Render a selected template with the project's theme or a default one.

    Parameters:
        request:
        template_name:
        modals:
        assets:
        context:
        use_theme:

    Returns:
        HTTPResponse
    """
    return _render(request,
                   template_name,
                   "core_main_app/_render/user/app_wrapper.html",
                   modals,
                   assets,
                   context,
                   use_theme)


def admin_render(request, template_name, modals=None, assets=None, context=None):
    """Render a selected template with the project's theme,

    Parameters:
        request:
        template_name:
        modals:
        assets:
        context:

    Returns:
        HTTPResponse
    """
    return _render(request, template_name, "core_main_app/_render/admin/app_wrapper.html", modals, assets, context)



""" Integration Test for User Rest API
"""
from rest_framework.status import HTTP_404_NOT_FOUND

from core_main_app.utils.integration_tests.integration_base_transaction_test_case import \
    MongoIntegrationTransactionTestCase
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from tests.components.user.fixtures.fixtures import UserFixtures
from core_main_app.rest.user import views as user_rest_views
from core_main_app.utils.integration_tests.integration_base_test_case import \
    MongoIntegrationBaseTestCase

from core_main_app.utils.tests_tools.RequestMock import RequestMock

user_fixture = UserFixtures()


class TestUserDetail(MongoIntegrationTransactionTestCase):

    def setUp(self):
        super(TestUserDetail, self).setUp()

    def test_get_returns_correct_user(self):
        # Arrange
        user = user_fixture.create_user()
        mock_user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_get(user_rest_views.UserDetail.as_view(),
                                              mock_user,
                                              param={'pk':user.id})

        # Assert
        self.assertEqual(user.id, response.data['id'])

    def test_get_returns_incorrect_user_raises_404(self):
        # Arrange
        user = user_fixture.create_user()
        mock_user = create_mock_user('1', is_staff=True)

        # Act
        response = RequestMock.do_request_get(user_rest_views.UserDetail.as_view(),
                                              mock_user,
                                              param={'pk': user.id})

        # Assert
        self.assertNotEqual(response.status_code, HTTP_404_NOT_FOUND)



""" Workspace API
"""
from core_main_app import settings
from core_main_app.access_control.decorators import access_control
from core_main_app.commons import exceptions
from core_main_app.components.group import api as group_api
from core_main_app.components.user import api as user_api
from core_main_app.components.workspace import access_control as workspace_api_access_control
from core_main_app.components.workspace.models import Workspace
from core_main_app.permissions import api as permission_api


def create_and_save(title, owner_id=None, is_public=False):
    """ Create and save a workspace. It will also create permissions.

    Args:
        owner_id
        title
        is_public

    Returns:
    """

    # Create workspace
    workspace = _create_workspace(title, owner_id, is_public)

    try:
        return workspace.save()
    except Exception as ex:
        # Rollback permissions
        permission_api.delete_permission(workspace.read_perm_id)
        permission_api.delete_permission(workspace.write_perm_id)
        raise exceptions.ModelError(str(ex))


def _create_workspace(title, owner_id=None, is_public=False):
    """ Create workspace.

    Args:
        title
        owner_id
        is_public

    Returns:
    """
    if owner_id is None:
        return Workspace(title=title,
                         read_perm_id=str(permission_api.create_read_perm(title, '').id),
                         write_perm_id=str(permission_api.create_write_perm(title, '').id),
                         is_public=is_public)
    else:
        return Workspace(title=title,
                         owner=str(owner_id),
                         read_perm_id=str(permission_api.create_read_perm(title, str(owner_id)).id),
                         write_perm_id=str(permission_api.create_write_perm(title, str(owner_id)).id),
                         is_public=is_public)


def set_title(workspace, new_title):
    """ Set the workspace's title.

    Args:
        workspace
        new_title

    Returns:
    """
    workspace.title = new_title
    workspace.save()


def get_all():
    """ Get all workspace.

    Returns:

    """
    return Workspace.get_all()


def get_all_by_owner(user):
    """ Get all workspaces created by the given user.

    Args:
        user

    Returns:

    """
    return Workspace.get_all_by_owner(str(user.id))


def get_by_id(workspace_id):
    """ Return the workspace with the given id.

    Args:
        workspace_id

    Returns:
        Workspace (obj): Workspace object with the given id

    """
    return Workspace.get_by_id(workspace_id)


def get_by_id_list(list_workspace_id):
    """ Return a list of workspaces with the given id list.

    Args:
        list_workspace_id

    Returns:
    """
    list_workspace = []
    for workspace_id in list_workspace_id:
        list_workspace.append(Workspace.get_by_id(workspace_id))
    return list_workspace


def get_all_workspaces_with_read_access_by_user(user):
    """ Get all workspaces with read access for the given user.

    Args:
        user

    Returns:

    """
    read_permissions = permission_api.get_all_workspace_permissions_user_can_read(user)
    return Workspace.get_all_workspaces_with_read_access_by_user_id(user.id, read_permissions)


def get_all_workspaces_with_write_access_by_user(user):
    """ Get all workspaces with write access for the given user.

    Args:
        user

    Returns:

    """
    write_permissions = permission_api.get_all_workspace_permissions_user_can_write(user)
    return Workspace.get_all_workspaces_with_write_access_by_user_id(user.id, write_permissions)


def get_all_workspaces_with_read_access_not_owned_by_user(user):
    """ Get the all workspaces with read access not owned by the given user.

    Args:
        user

    Returns:

    """
    read_permissions = permission_api.get_all_workspace_permissions_user_can_read(user)
    return Workspace.get_all_workspaces_with_read_access_not_owned_by_user_id(user.id, read_permissions)


def get_all_workspaces_with_write_access_not_owned_by_user_id(user):
    """ Get the all workspaces with write access not owned by the given user.

    Args:
        user

    Returns:

    """
    write_permissions = permission_api.get_all_workspace_permissions_user_can_write(user)
    return Workspace.get_all_workspaces_with_write_access_not_owned_by_user_id(user.id, write_permissions)


def get_all_public_workspaces():
    """ Get all public workspaces.

    Args:

    Returns:

    """
    return Workspace.get_all_public_workspaces()


def get_all_other_public_workspaces(user):
    """ Get all other public workspaces.

    Args:
        user
    Returns:

    """
    return Workspace.get_all_other_public_workspaces(user.id)


def get_non_public_workspace_owned_by_user(user):
    """ Get the non public workspaces owned by the given user.

    Args:
        user:

    Returns:

    """
    return Workspace.get_non_public_workspace_owned_by_user_id(user.id)


def get_public_workspaces_owned_by_user(user):
    """ Get the public workspaces owned the given user.

    Args:
        user

    Returns:

    """
    return Workspace.get_public_workspaces_owned_by_user_id(user.id)


def is_workspace_public(workspace):
    """ Check if the workspace is public.

    Args:
        workspace

    Return:
    """
    return workspace.is_public


def is_workspace_global(workspace):
    """ Check if the workspace is global public.

    Args:
        workspace

    Return:
    """
    return workspace.is_global


def get_global_workspace():
    """ Get global workspace.

    Return:
    """
    return Workspace.get_global_workspace()


def can_user_read_workspace(workspace, user):
    """ Check if user has read permission on workspace.

    Args:
        workspace
        user

    Return:
    """
    if is_workspace_public(workspace):
        return True
    permission_label = permission_api.get_permission_label(workspace.read_perm_id)
    return str(workspace.owner) == str(user.id) or user.has_perm(permission_label)


def can_user_write_workspace(workspace, user):
    """ Check if user has write permission on workspace.

    Args:
        workspace
        user

    Return:
    """
    permission_label = permission_api.get_permission_label(workspace.write_perm_id)
    return str(workspace.owner) == str(user.id) or user.has_perm(permission_label)


def can_group_read_workspace(workspace, group):
    """ Check if group has read permission on workspace.

    Args:
        workspace
        group

    Return:
    """
    if is_workspace_public(workspace):
        return True
    permission = permission_api.get_by_id(workspace.read_perm_id)
    return permission_api.check_if_group_has_perm(group, permission)


def can_group_write_workspace(workspace, group):
    """ Check if group has write permission on workspace.

    Args:
        workspace
        group

    Return:
    """
    permission = permission_api.get_by_id(workspace.write_perm_id)
    return permission_api.check_if_group_has_perm(group, permission)


def get_list_group_can_access_workspace(workspace, user):
    """ Get the list of groups that have either read or write access to workspace.

    Args:
        workspace
        user

    Returns:
    """

    # List all groups that have the read permission of the workspace
    all_groups_read = get_list_group_can_read_workspace(workspace, user)

    # List all groups that have the write permission of the workspace
    all_groups_write = get_list_group_can_write_workspace(workspace, user)

    # Return the union without doublons of the two lists.
    return list(set(all_groups_read + all_groups_write))


def get_list_user_can_access_workspace(workspace, user):
    """ Get the list of users that have either read or write access to workspace.

    Args:
        workspace
        user

    Returns:
    """

    # List all users that have the read permission of the workspace
    all_users_read = get_list_user_can_read_workspace(workspace, user)

    # List all users that have the write permission of the workspace
    all_users_write = get_list_user_can_write_workspace(workspace, user)

    # Return the union without doublons of the two lists.
    return list(set(all_users_read + all_users_write))


def check_if_workspace_can_be_changed(document,
                                      allow_change_workspace_if_public=settings.CAN_SET_PUBLIC_DATA_TO_PRIVATE):
    """ Check if a workspace of a document can be changed

    Args:
        document:
        allow_change_workspace_if_public:

    Returns:

    """
    workspace = document.workspace
    if workspace is not None and is_workspace_public(workspace) and not allow_change_workspace_if_public:
        return False
    return True


@access_control(workspace_api_access_control.can_user_set_workspace_public)
def set_workspace_public(workspace, user):
    """ Set the workspace to public.

    Args:
        workspace
        user

    Return:
    """
    if settings.CAN_SET_WORKSPACE_PUBLIC:
        workspace.is_public = True
        workspace.save()
    else:
        raise exceptions.ApiError("You can't change the state of the workspace because of the settings of the website.")


@access_control(workspace_api_access_control.is_workspace_owner)
def set_workspace_private(workspace, user):
    """ Set the workspace to private.

    Args:
        workspace
        user

    Return:
    """
    if is_workspace_global(workspace):
        raise exceptions.ApiError("You can't change the state of the global workspace.")

    if settings.CAN_SET_PUBLIC_DATA_TO_PRIVATE:
        workspace.is_public = False
        workspace.save()
    else:
        raise exceptions.ApiError("You can't change the state of the workspace because of the settings of the website.")


@access_control(workspace_api_access_control.can_delete_workspace)
def delete(workspace, user):
    """ Delete a workspace and its permissions.

    Args:
         workspace:

    Returns:
    """

    # Can't delete a global workspace
    if workspace.is_global:
        raise exceptions.ModelError("The global workspace can not be deleted.")

    permission_api.delete_permission(workspace.read_perm_id)
    permission_api.delete_permission(workspace.write_perm_id)
    workspace.delete()


@access_control(workspace_api_access_control.is_workspace_owner)
def get_list_user_can_write_workspace(workspace, user):
    """ Get list of users that have write access to workspace.

    Args:
        workspace
        user

    Return:
    """
    # Get write permission of the workspace
    write_permission = permission_api.get_by_id(workspace.write_perm_id)
    return list(write_permission.user_set.all())


@access_control(workspace_api_access_control.is_workspace_owner)
def get_list_user_can_read_workspace(workspace, user):
    """  Get list of users that have read access to workspace.

    Args:
        workspace
        user

    Return:
    """

    if is_workspace_public(workspace):
        return list(user_api.get_all_users())

    # Get read permission of the workspace
    read_permission = permission_api.get_by_id(workspace.read_perm_id)

    return list(read_permission.user_set.all())


@access_control(workspace_api_access_control.is_workspace_owner)
def get_list_user_with_no_access_workspace(workspace, user):
    """ Get list of users that don't have any access to the workspace.

    Args:
         workspace
         user

    Returns:
    """
    return user_api.get_all_users_except_list(get_list_user_can_access_workspace(workspace, user))


@access_control(workspace_api_access_control.is_workspace_owner_to_perform_action_for_others)
def add_user_read_access_to_workspace(workspace, new_user, user):
    """ Add to new user the read access to workspace.

    Args:
          workspace
          new_user
          user
    Returns:
    """
    permission_api.add_permission_to_user(new_user, workspace.read_perm_id)


@access_control(workspace_api_access_control.is_workspace_owner_to_perform_action_for_others)
def add_user_write_access_to_workspace(workspace, new_user, user):
    """ Add to new user the write access to workspace.

    Args:
          workspace
          new_user
          user
    Returns:
    """
    if is_workspace_global(workspace):
        raise exceptions.ModelError("You can't modify the rights of the global public workspace.")
    permission_api.add_permission_to_user(new_user, workspace.write_perm_id)


@access_control(workspace_api_access_control.is_workspace_owner_to_perform_action_for_others)
def remove_user_read_access_to_workspace(workspace, new_user, user):
    """ Remove to new user the read access to workspace.

    Args:
          workspace
          new_user
          user
    Returns:
    """
    permission_api.remove_permission_to_user(new_user, workspace.read_perm_id)


@access_control(workspace_api_access_control.is_workspace_owner_to_perform_action_for_others)
def remove_user_write_access_to_workspace(workspace, new_user, user):
    """ Remove to new user the write access to workspace.

    Args:
          workspace
          new_user
          user
    Returns:
    """
    if is_workspace_global(workspace):
        raise exceptions.ModelError("You can't modify the rights of the global public workspace.")
    permission_api.remove_permission_to_user(new_user, workspace.write_perm_id)


@access_control(workspace_api_access_control.is_workspace_owner)
def get_list_group_can_write_workspace(workspace, user):
    """ Get the list of groups that have write access to workspace.

    Args:
        workspace
        user

    Returns:
    """
    # Get write permission of the workspace
    write_permission = permission_api.get_by_id(workspace.write_perm_id)

    return list(write_permission.group_set.all())


@access_control(workspace_api_access_control.is_workspace_owner)
def get_list_group_can_read_workspace(workspace, user):
    """ Get the list of groups that have read access to workspace.

    Args:
        workspace
        user

    Returns:
    """
    if is_workspace_public(workspace):
        return list(group_api.get_all_groups())

    # Get read permission of the workspace
    read_permission = permission_api.get_by_id(workspace.read_perm_id)

    return list(read_permission.group_set.all())


@access_control(workspace_api_access_control.is_workspace_owner)
def get_list_group_with_no_access_workspace(workspace, user):
    """ Get list of groups that don't have any access to the workspace.

    Args:
         workspace
         user

    Returns:
    """
    return group_api.get_all_groups_except_list(get_list_group_can_access_workspace(workspace, user))


@access_control(workspace_api_access_control.is_workspace_owner_to_perform_action_for_others)
def add_group_read_access_to_workspace(workspace, new_group, user):
    """ Add to new group the read access to workspace.

    Args:
          workspace
          new_group
          user
    Returns:
    """
    permission_api.add_permission_to_group(new_group, workspace.read_perm_id)


@access_control(workspace_api_access_control.is_workspace_owner_to_perform_action_for_others)
def add_group_write_access_to_workspace(workspace, new_group, user):
    """ Add to new group the write access to workspace.

    Args:
          workspace
          new_group
          user
    Returns:
    """
    if is_workspace_global(workspace):
        raise exceptions.ModelError("You can't modify the rights of the global public workspace.")
    permission_api.add_permission_to_group(new_group, workspace.write_perm_id)


@access_control(workspace_api_access_control.is_workspace_owner_to_perform_action_for_others)
def remove_group_read_access_to_workspace(workspace, group, user):
    """ Remove to new group the read access to workspace.

    Args:
          workspace
          group
          user
    Returns:
    """
    permission_api.remove_permission_to_group(group, workspace.read_perm_id)


@access_control(workspace_api_access_control.is_workspace_owner_to_perform_action_for_others)
def remove_group_write_access_to_workspace(workspace, group, user):
    """ Remove to new group the write access to workspace.

    Args:
          workspace
          group
          user
    Returns:
    """
    if is_workspace_global(workspace):
        raise exceptions.ModelError("You can't modify the rights of the global public workspace.")
    permission_api.remove_permission_to_group(group, workspace.write_perm_id)



"""
Version Manager model
"""
from mongoengine import errors as mongoengine_errors

from core_main_app.commons import exceptions
from core_main_app.utils.validation.regex_validation import not_empty_or_whitespaces
from django_mongoengine import fields, Document


# TODO: could change versions to ReferenceField (Document?)
# TODO: could make current an IntField (index of the current in versions)
# TODO: could make is_disabled a Status with other possible values taken from an enum


class VersionManager(Document):
    """Version Manager"""
    title = fields.StringField(unique=True, validation=not_empty_or_whitespaces)
    user = fields.StringField(blank=True)
    versions = fields.ListField(default=[], blank=True)
    current = fields.StringField(blank=True)
    is_disabled = fields.BooleanField(default=False)
    disabled_versions = fields.ListField(default=[], blank=True)

    meta = {'allow_inheritance': True}

    def disable(self):
        """Disable the Version Manager.

        Returns:

        """
        self.is_disabled = True

    def restore(self):
        """Restore the Version Manager.

        Returns:

        """
        self.is_disabled = False

    def disable_version(self, version):
        """Disable a version.

        Args:
            version:

        Returns:

        """
        self.disabled_versions.append(str(version.id))

    def restore_version(self, version):
        """Restore a version.

        Args:
            version:

        Returns:

        """
        self.disabled_versions.remove(str(version.id))

    def set_current_version(self, version):
        """Set the current version.

        Args:
            version:

        Returns:

        """
        self.current = str(version.id)

    def get_version_number(self, version_id):
        """Return version number from version id.

        Args:
            version_id:

        Returns:

        Raises:
            DoesNotExist: Version does not exist.

        """
        try:
            return self.versions.index(str(version_id)) + 1
        except Exception as e:
            raise exceptions.DoesNotExist(str(e))

    def insert(self, version):
        """Insert a version in the Version Manager.

        Args:
            version:

        Returns:

        """
        self.versions.append(str(version.id))

    def get_disabled_versions(self):
        """Get the list disabled versions of the version manager.

        Returns:

        """
        return self.disabled_versions

    def get_version_by_number(self, version_number):
        """Return the version by its version number.

        Args:
            version_number: Number of the version.

        Returns:

        Raises:
            DoesNotExist: Version does not exist.

        """
        try:
            return self.versions[version_number - 1]
        except Exception as e:
            raise exceptions.DoesNotExist(str(e))

    @staticmethod
    def get_all():
        """Return all Version Managers.

        Returns:

        """
        return VersionManager.objects.all()

    @staticmethod
    def get_by_id(version_manager_id):
        """Return Version Managers by id.

        Args:
            version_manager_id:

        Returns:

        """
        try:
            return VersionManager.objects.get(pk=str(version_manager_id))
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(str(e))
        except Exception as e:
            raise exceptions.ModelError(str(e))

    @staticmethod
    def get_by_id_list(list_id):
        """Return Version Managers with the given id list.

        Args:
            list_id:

        Returns:

        """
        return VersionManager.objects(pk__in=list_id).all()

    @staticmethod
    def get_active_global_version_manager_by_title(version_manager_title):
        """Return active Version Manager by its title with user set to None.

        Args:
            version_manager_title: Version Manager title

        Returns:
            Version Manager instance

        """
        try:
            return VersionManager.objects.get(is_disabled=False, title=version_manager_title, user=None)
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(str(e))
        except Exception as e:
            raise exceptions.ModelError(str(e))

    @staticmethod
    def get_global_version_managers():
        """Return all Version Managers with user set to None.

        Returns:

        """
        return VersionManager.objects(user=None).all()

    @staticmethod
    def get_active_global_version_manager():
        """ Return all active Version Managers with user set to None.

        Returns:

        """
        return VersionManager.objects(is_disabled=False, user=None).all()

    @staticmethod
    def get_disable_global_version_manager():
        """ Return all disabled Version Managers with user set to None.

        Returns:

        """
        return VersionManager.objects(is_disabled=True, user=None).all()

    @staticmethod
    def get_active_version_manager_by_user_id(user_id):
        """ Return all active Version Managers with given user id.

        Returns:

        """
        return VersionManager.objects(is_disabled=False, user=str(user_id)).all()

    @staticmethod
    def get_disable_version_manager_by_user_id(user_id):
        """ Return all disabled Version Managers with given user id.

        Returns:

        """
        return VersionManager.objects(is_disabled=True, user=str(user_id)).all()

    @staticmethod
    def get_all_version_manager_except_user_id(user_id):
        """ Return all Version Managers of all users except user with given user id.

        Args:
            user_id: user_id.

        Returns:

        """
        return VersionManager.objects(user__nin=str(user_id)).all()

    @staticmethod
    def get_all_version_manager_by_user_id(user_id):
        """ Return all Version Managers with given user id.

        Args:
            user_id: user_id.

        Returns:

        """
        return VersionManager.objects(user=str(user_id)).all()

    def save_version_manager(self):
        """ Custom save.

        Returns:
            Saved Instance.

        """
        try:
            return self.save()
        except mongoengine_errors.NotUniqueError as e:
            raise exceptions.NotUniqueError(str(e))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    def clean(self):
        """ Clean is called before saving

        Returns:

        """
        self.title = self.title.strip()



""" Serializers for Django Group
"""
from django.contrib.auth.models import Group
from rest_framework import serializers


class GroupSerializer(serializers.ModelSerializer):
    """ Group serializer
    """
    class Meta(object):
        """ Meta
        """
        model = Group
        fields = ('id', 'name')


from unittest import TestCase

from django.urls import reverse
from django.test.utils import override_settings
from mock.mock import patch

from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.template import api as template_api
from core_main_app.components.template.models import Template
from core_main_app.utils.xsd_flattener.xsd_flattener_database_url import \
    XSDFlattenerDatabaseOrURL, XSDFlattenerRequestsURL


class TestXSDFlattenerDatabaseUrl(TestCase):
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    @patch.object(XSDFlattenerRequestsURL, 'get_dependency_content')
    def test_url_not_recognized_use_xsd_flattener_url(self, mock_get_dependency_content):
        # Arrange
        xml_string = '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">' \
                     '<xs:include schemaLocation="http://dummy.com/download?id=1234"/></xs:schema>'
        dependency = '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">' \
                     '<xs:element name="test"/></xs:schema>'
        mock_get_dependency_content.return_value = dependency

        # Act
        flattener = XSDFlattenerDatabaseOrURL(xml_string)
        flat_string = flattener.get_flat()

        # Assert
        self.assertTrue('<xs:element name="test"/>' in flat_string)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    @patch.object(template_api, 'get')
    def test_url_recognized_use_database(self, mock_get):
        # Arrange
        url_template_download = reverse('core_main_app_rest_template_download', kwargs={'pk': 'pk'})
        xml_string = '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">' \
                     '<xs:include schemaLocation="http://dummy.com{0}?id=1234"/>' \
                     '</xs:schema>'.format(url_template_download)
        dependency = '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">' \
                     '<xs:element name="test"/></xs:schema>'
        mock_get.return_value = Template(content=dependency)

        # Act
        flattener = XSDFlattenerDatabaseOrURL(xml_string)
        flat_string = flattener.get_flat()

        # Assert
        self.assertTrue('<xs:element name="test"/>' in flat_string)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    @patch.object(template_api, 'get')
    @patch.object(XSDFlattenerRequestsURL, 'get_dependency_content')
    def test_url_recognized_template_does_not_exist(self, mock_get_dependency_content, mock_get):
        # Arrange
        url_template_download = reverse('core_main_app_rest_template_download', kwargs={'pk': 'pk'})
        xml_string = '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">' \
                     '<xs:include schemaLocation="http://dummy.com{0}?id=1234"/>' \
                     '</xs:schema>'.format(url_template_download)
        dependency = '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">' \
                     '<xs:element name="test"/></xs:schema>'
        mock_get_dependency_content.return_value = dependency
        mock_get.side_effect = DoesNotExist("Error")

        # Act
        flattener = XSDFlattenerDatabaseOrURL(xml_string)
        flat_string = flattener.get_flat()

        # Assert
        self.assertTrue('<xs:element name="test"/>' in flat_string)



""" Set of functions to define the rules for access control
"""
import logging

import core_main_app.permissions.rights as rights
from core_main_app.access_control.api import has_perm_publish, can_write_in_workspace

logger = logging.getLogger(__name__)


def has_perm_publish_blob(user):
    """ Does the user have the permission to publish a blob.

    Args:
        user

    Returns
    """
    has_perm_publish(user, rights.publish_blob)


def can_write_blob_workspace(func, data, workspace, user):
    """ Can user write data in workspace.

    Args:
        func:
        data:
        workspace:
        user:

    Returns:

    """
    return can_write_in_workspace(func, data, workspace, user, rights.publish_blob)


"""
    Serializers used throughout the Rest API
"""
from django.http import Http404
from rest_framework.fields import CharField
from rest_framework.fields import FileField, SerializerMethodField
from rest_framework_mongoengine.serializers import DocumentSerializer

import core_main_app.components.blob.api as blob_api
from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.blob.models import Blob
from core_main_app.components.blob.utils import get_blob_download_uri
from core_main_app.access_control.exceptions import AccessControlError


class BlobSerializer(DocumentSerializer):
    """ Blob serializer
    """
    blob = FileField(write_only=True)
    handle = SerializerMethodField()
    upload_date = SerializerMethodField()

    class Meta(object):
        model = Blob
        fields = ['id',
                  'user_id',
                  'filename',
                  'handle',
                  'blob',
                  'upload_date']
        read_only_fields = ('id', 'user_id', 'filename', 'handle', 'upload_date',)

    def get_handle(self, instance):
        """ Return handle

        Args:
            instance:

        Returns:

        """
        # get request from context
        request = self.context.get('request')
        # return download handle
        return get_blob_download_uri(instance, request)

    def get_upload_date(self, instance):
        """ Return upload date

        Args:
            instance:

        Returns:

        """
        # Return instance generation time
        return str(instance.id.generation_time)

    def create(self, validated_data):
        """ Create and return a new `Blob` instance, given the validated data.

        Args:
            validated_data:

        Returns:

        """
        # Create blob
        blob_object = Blob(filename=validated_data['blob'].name,
                           user_id=str(validated_data['user'].id))
        # Set file content
        blob_object.blob = validated_data['blob'].file

        # Save the blob
        return blob_api.insert(blob_object)


class DeleteBlobsSerializer(DocumentSerializer):
    """ Delete Blob serializer.
    """
    id = CharField()

    class Meta(object):
        model = Blob
        fields = ('id', )

    def validate_id(self, id):
        """ Validate id field

        Args:
            id:

        Returns:

        """
        request = self.context.get('request')
        try:
            blob_api.get_by_id(id, request.user)
        except DoesNotExist:
            raise Http404
        return id
